﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HamiltoninaGSD
{
    enum CycleStepResult {
        AddedNode, CompletedCycle, Failed, Continue            
    }
   
    class BFSGraph: ABSGraph
    {


        protected BFSNode[]             mNodes;
       
//        protected List<List<int>>       mBFSFoundEars;
        protected BFSEarContainer mBFSFoundEars;


        bool mFinished;


        protected int               mCounter;
        protected Queue<BFSNode>    mQueue;
        protected BFSNode           mCurrentTest;
        protected BFSNode           mLastUsed;
        protected int               mRoot;
        protected Random            mGenerator;
        protected int               mNumberOfNodesInCycle;
        protected bool              mUpdateNumberOfNodesInCylce;
        protected int               mMaxNodesInCycles;
        public List<int>            mCurrentNeighbors;

        public bool Finished {
            get { return mFinished; }
            set { mFinished  = value; }
        }
        public bool IsComplete() {

            return mMaxNodesInCycles == 1+NumberOfCycleNodes();

        }
        public bool CanAdd(int weight) {

            if (NumberOfCycleNodes() + weight < mMaxNodesInCycles)
                return true;
            return false;
        }
        public int Root
        {

            get { return mRoot; }
            set { mRoot = value; }
        }
        public BFSEar GetLastEar() {

            return mBFSFoundEars.GetLastEar();
        }
        public List<int> GetDescendants(int Index) {

            return mNodes[Index].BFSDescendants;
        }

        /*
        mSubGraphs[ss[2]].GetNode(tn).BFSInCycle = true;
                         

                         mSubGraphs[ss[2]].GetNode(tn).BFSTreeRootParent = tn;
                         mSubGraphs[ss[2]].GetNode(tn).BFSInCycle = true;
                         mSubGraphs[ss[2]].GetNode(tn).BFSDistance = 0;


                         mSubGraphs[ss[2]].AddEar(tn, ss[0], ss[1]);

                        if (mSubGraphs[ss[2]].GetNode(tn).EarIndex == -1)
                                mSubGraphs[ss[2]].GetNode(tn).EarIndex
                                */
        public void SetNodeBFSInCycle(int index, bool vBFSInCycle) { mNodes[index].BFSInCycle = vBFSInCycle; }
        public void SetNodeBFSTreeRootParent(int index,int vBFSTreeRootParent) { mNodes[index].BFSTreeRootParent = vBFSTreeRootParent; }
        public void SetNodeBFSDistance(int index, int vBFSDistance) { mNodes[index].BFSDistance = vBFSDistance; }
        public void SetNodeBFSEarIndex(int index, int vEarIndex) { mNodes[index].EarIndex = vEarIndex; }


        public void MakeUnavailable(bool[] iUnavailable) {

            for (int i = 0; i < mNumberOfVertices; i++) {
                if ((iUnavailable[i] == true)&&(i != mRoot))
                    mNodes[i].BFSAvailable = false;
                else
                    mNodes[i].BFSAvailable = true;

            }
        }

        public void Restart() {

            mRoot = 0; mFinished = false;
            mBFSFoundEars = new BFSEarContainer();
            mBFSFoundEars.SetGraph(this);
            mNumberOfNodesInCycle = 0;
            InitMaxNodesInCycles();

        }
        public int RemoveNodeFromCycles(int NodeIndex) {
            if (!mNodes[NodeIndex].BFSInCycle)
                return 0;
            BFSEar cEar = mBFSFoundEars.GetEar(mNodes[NodeIndex].EarIndex);

            if (cEar != null)
                return RemoveEarFromCycles(cEar);
            else
                return 0;

            return 0;

        }
        public int RemoveEarFromCycles(BFSEar rEar)
        {
            int result = 0;
            int cNode;

            mUpdateNumberOfNodesInCylce = true;
            result = rEar.Nodes.Count - 2;
            //            foreach (int cNode in rEar.Nodes) {
            for (int i = 1; i < rEar.Nodes.Count - 1; i++) { 
                cNode = rEar.Nodes[i];
                mNodes[cNode].BFSInCycle = false;
                mNodes[cNode].BFSDistance = -1;
                mNodes[cNode].BFSFinished = false;
                mNodes[cNode].BFSAvailable = true;
                mNodes[cNode].EarIndex = -1;
                mNodes[cNode].BFSParent = -1;
                mNodes[cNode].BFSTreeRoot = -1;
                mNodes[cNode].BFSTreeRootParent = -1;
            }

            BFSEar cEar;
            List<int> gDescendants = new List<int>(rEar.Descendants); 
            foreach (int desc in gDescendants) {
                cEar = mBFSFoundEars.GetEar(desc);
                if(cEar!= null)
                    result += RemoveEarFromCycles(cEar);
                
            }
            mBFSFoundEars.RemoveEar(rEar.Id);
            return result;
        }

        public int NumberOfCycleNodes() {

            int results = 0;

            if(!mUpdateNumberOfNodesInCylce)
                return mNumberOfNodesInCycle;

            for (int i = 0; i < mNumberOfVertices; i++)
                if (mNodes[i].BFSInCycle)
                    results++;


            mUpdateNumberOfNodesInCylce = false;
            mNumberOfNodesInCycle = results;
            return mNumberOfNodesInCycle;
        }
        public Random Generator{
            get { return mGenerator; }
            set { mGenerator = value; }
            }
        public BFSEarContainer GetEars() { return mBFSFoundEars; }

        public int GetBFS(int index) { return mNodes[index].BFSCounter; }
        public int GetBFSParent(int index) { return mNodes[index].BFSParent; }
        public int GetBFSDistance(int index) { return mNodes[index].BFSDistance; }
//        public int GetBFSTreeRoot(int index) { return mBFSTreeRoot[index]; }
        public int GetBFSTreeRoot(int index) {

            if (mNodes[index].BFSTreeRootParent == -1)
                return mNodes[index].BFSTreeRootParent;
            if (mNodes[index].BFSTreeRootParent == index)
                return mNodes[index].BFSTreeRootParent;
            return GetBFSTreeRoot(mNodes[index].BFSTreeRootParent);

        }
        
        public int GetBFSTreeRootParent(int index) { return mNodes[index].BFSTreeRootParent; }
        public bool IsInCycle(int index) { return mNodes[index].BFSInCycle; }
        public bool IsFinished(int index) { return mNodes[index].BFSFinished; }
        public bool IsCurrent(int Index) {
            if (mCurrentTest != null)
//                if (mCurrentTest.Index != -1)
                    return mCurrentTest.Index == Index;
            else
                return false;
        }

        public void DeleteTreeNode(int index, bool root, bool Available= false ) {

            if (mNodes[index].BFSInCycle)
                return;
            if (root)
            {
                mNodes[index].BFSAvailable = false;
                if (mNodes[index].BFSParent != -1)
                    //                mBFSDescendants[mBFSParent[index]].Remove(index);
                    if(mNodes[mNodes[index].BFSParent].BFSDescendants != null)
                            mNodes[mNodes[index].BFSParent].BFSDescendants.Remove(index);

            }
            //            mBFSFinished[index] = false;
            mNodes[index].BFSFinished = false;
            //            if(mBFSDescendants[index]!= null)
             if(mNodes[index].BFSDescendants!= null)

            foreach (int desc in mNodes[index].BFSDescendants) {

                if (!mNodes[index].BFSInCycle)
                {

                        mNodes[desc].BFSAvailable = Available;
                        mNodes[desc].BFSDistance = -1;
                        mNodes[desc].BFSFinished = false;
                    DeleteTreeNode(desc, false);


/*                        [index]mBFSAvailable[desc] = Available;
                    mBFSDistance[desc] = -1;
                    mBFSFinished[desc] = false;
                    DeleteTreeNode(desc, false);
                    */
                    }

                }
             if (mNodes[index].BFSDescendants!= null) 
                    mNodes[index].BFSDescendants.Clear();
               


        }
        public bool IsLastUsed(int Index)
        {
            if (mLastUsed != null)
//                if (mLastUsed.Index != -1)
                    return mLastUsed.Index == Index;
            else
                return false;
        }

        public BFSNode GetNode(int index) { return mNodes[index]; }

        public BFSGraph() : base() { mRoot = 0;mFinished = false; InitMaxNodesInCycles(); }
        public BFSGraph(int NumberOfVertex) : base(NumberOfVertex) { mRoot = 0; mFinished = false; InitMaxNodesInCycles(); }
        public BFSGraph(ABSGraph A) : base(A) {
            mRoot = 0; mFinished = false;
            mBFSFoundEars = new BFSEarContainer();
            mBFSFoundEars.SetGraph(A);
            mNumberOfNodesInCycle = 0;
            InitMaxNodesInCycles();
        }

        public void InitMaxNodesInCycles()
        {
            mMaxNodesInCycles = 50;
        }
        //        public BFSGraph(BFSGraph A) : base((ABSGraph)A) {

         public BFSGraph(BFSGraph A, bool All = true) {

            mNumberOfVertices = A.mNumberOfVertices;
            mNumberOfEdges = A.mNumberOfEdges;
            mEdges = A.mEdges;

            mBFSFoundEars = new BFSEarContainer();
            mBFSFoundEars.SetGraph(A);
            InitBFSEnumeration(false);
            InitMaxNodesInCycles();
            mRoot = A.mRoot;

            mCounter = A.mCounter;
            mCurrentTest = null;
//            mCurrentTest.Index = -1;
            mFinished = false;





            mNodes = new BFSNode[A.mNumberOfVertices];
//            mNodes = (BFSNode[])A.mNodes.Clone();
            if (!All)
            {
                A.mNodes.CopyTo(mNodes, 0);
            }
            else
            {
             
                for (int i = 0; i < mNumberOfVertices; i++) {

                    mNodes[i] = new BFSNode(A.GetNode(i), All);
                }
             
            }
            mBFSFoundEars.SetGraph(A);
            if (All)
            {
                
//                List<int> Current;

                if (A.mBFSFoundEars != null)
                {
                    mBFSFoundEars = new BFSEarContainer();
                    foreach (BFSEar ear in A.mBFSFoundEars.Ears)
                    {

                        mBFSFoundEars.Add(new BFSEar(ear));

                    }
                }
            }
            else {
                mBFSFoundEars = null;

            }
            mUpdateNumberOfNodesInCylce = true;
            
        }


        public void InitBFSEnumeration(bool Full = true) {
            mCounter = 0;
            mQueue = new Queue<BFSNode>();
            mBFSFoundEars = new BFSEarContainer();


            if (Full)
            {
//                if(mNodes == null)
                mNodes = new BFSNode[mNumberOfVertices];

                for (int i = 0; i < mNumberOfVertices; i++)
                {

                    //                mNodes[i] = new BFSNode();
                    if (mNodes[i] == null)
                        mNodes[i] = new BFSNode(i);
                    else {

                    }
                }
            }
            /**/

            mBFSFoundEars = new BFSEarContainer();
            mUpdateNumberOfNodesInCylce = true;
        }
        public void StartEnumerate() {

            InitBFSEnumeration();

            mQueue.Enqueue(new BFSNode(mRoot));
            mLastUsed = mQueue.Peek();
            mNodes[mRoot].BFSCounter = mCounter;
            mNodes[mRoot].BFSDistance = 0;
            mNodes[mRoot].BFSParent = -1;
            mNodes[mRoot].BFSTreeRoot = -1;
            mNodes[mRoot].BFSTreeRootParent = -1;
            mNodes[mRoot].BFSInCycle = true;
            mNumberOfNodesInCycle = 1;
            mCounter++;
                        mCurrentTest = null;
//            mCurrentTest.Index = -1; 



        }

        public bool StepEnumerate()
        {

            BFSNode nCurrent;

            if (mQueue == null)
                return false;
            if (mQueue.Count == 0)
                return false;

            if(mCurrentTest == null)
//            if (mCurrentTest.Index == -1)

                    mCurrentTest = mQueue.Dequeue();

            
            foreach (int u in mEdges[mCurrentTest.Index]) {

                if (mNodes[u].BFSDistance == -1) {

                    mNodes[u].BFSDistance = mCurrentTest.BFSDistance + 1;
                    mNodes[u].BFSCounter  = mCounter++;
                    mNodes[u].BFSParent = mCurrentTest.Index;

                    mQueue.Enqueue(new BFSNode(u, mNodes[u].BFSCounter, mNodes[u].BFSDistance, mCurrentTest.Index));
                    return true;
                  
                 }
            }
                        mCurrentTest = null;
            //            mCurrentTest.Index = -1;

            //   mQueue.Enqueue(new BFSNode(0));
            //       mBFSCounter[0] = mCounter;


            return true;

        }


        public CycleStepResult StepEnumerateCycleStart() {

            if (mQueue == null)
            {
                mFinished = true;
                return CycleStepResult.Failed;
            }
            if ((mQueue.Count == 0) && (mCurrentTest == null))
//                if ((mQueue.Count == 0) && (mCurrentTest.Index == -1))

                {
                    mFinished = true;
                return CycleStepResult.Failed;
            }



            if (mCurrentTest == null)
//                if (mCurrentTest.Index == -1)
                {
                    mCurrentTest = mQueue.Dequeue();
                mCurrentNeighbors = mEdges[mCurrentTest.Index].ToList();
            }
             if (!mNodes[mCurrentTest.Index].BFSAvailable)
            {

//                mCurrentTest.Index = -1;
//                return CycleStepResult.Failed;

                mCurrentTest = null;
                return CycleStepResult.Failed;
            }

            if (mNodes[mCurrentTest.Index].BFSDistance == -1)
            {
                mCurrentTest = null;
//                mCurrentTest.Index = -1;
                return CycleStepResult.Failed;
            }


            mLastUsed = mCurrentTest;

            return CycleStepResult.Continue;
        }


        CycleStepResult CheckNode(int u) {


            if (!mNodes[u].BFSAvailable)
                return CycleStepResult.Continue;


            if (mNodes[u].BFSDistance == -1)
            {
                // new node
                mNodes[u].BFSDistance = mNodes[mCurrentTest.Index].BFSDistance + 1;
                mNodes[u].BFSCounter = mCounter++;
                mNodes[u].BFSParent = mCurrentTest.Index;
                if (mNodes[mCurrentTest.Index].BFSDescendants == null)
                    mNodes[mCurrentTest.Index].BFSDescendants = new List<int>(); 

                mNodes[mCurrentTest.Index].BFSDescendants.Add(u);
                if (GetBFSTreeRoot(mCurrentTest.Index) == -1)
                {
                    mNodes[u].BFSTreeRootParent = u;
                }
                else
                {
                    mNodes[u].BFSTreeRootParent = mCurrentTest.Index;

                }

                mQueue.Enqueue(new BFSNode(u, mNodes[u].BFSCounter, mNodes[u].BFSDistance, mCurrentTest.Index));
                return CycleStepResult.AddedNode;

            }
            else
            {
             //   if ((GetBFSTreeRoot(mCurrentTest.Index) != GetBFSTreeRoot(u)) && (mCurrentTest.Index != u) && (mBFSParent[u] != mCurrentTest.Index)
//                    && (mBFSParent[mCurrentTest.Index] != u))

//                    if (IsNewCycle(u, mCurrentTest.Index)&& (mGenerator.Next()%2 >0))
                    if (IsNewCycle(u, mCurrentTest.Index) && (mGenerator.Next() % 8 <3))
                    {
                        bool IsFirst;

//                    IsFirst = mNodes[mRoot].BFSTreeRootParent == -1;
                    IsFirst = GetBFSTreeRoot(mRoot) == -1;

                    AddEar(mCurrentTest.Index, u);
                    UpdateInsideCycle(mCurrentTest.Index, mBFSFoundEars.Ears.Last().Id);
                    UpdateInsideCycle(u, mBFSFoundEars.Ears.Last().Id);
                    if(IsFirst)
                        UpdateRootFirstCycle();

                    AddPotetnialRedo();
                    mQueue.Enqueue(new BFSNode(u, mCurrentTest.Index, mNodes[mCurrentTest.Index].BFSDistance, mNodes[mCurrentTest.Index].BFSParent));
//                    RemoveEdge(mCurrentTest.Index, u);
                    mCurrentNeighbors.Remove(u);
                    return CycleStepResult.CompletedCycle;
                }


            }


            return CycleStepResult.Continue;
        }

        public virtual bool IsAvailable(int index) {
            return mNodes[index].BFSAvailable;
        }

        public virtual bool IsNewCycle(int Index, int ParentIndex)
        {

            int pBFSTreeRoot = GetBFSTreeRoot(ParentIndex);
            int iBFSTreeRoot = GetBFSTreeRoot(Index);


            return (pBFSTreeRoot != iBFSTreeRoot) && (ParentIndex != Index) && (mNodes[Index].BFSParent != ParentIndex)
                && (mNodes[ParentIndex].BFSParent != Index) && (NumberOfCycleNodes() + mNodes[ParentIndex].BFSDistance + mNodes[Index].BFSDistance < mMaxNodesInCycles)
                && (!(mNodes[ParentIndex].BFSInCycle && mNodes[Index].BFSInCycle));
            /*

            return (GetBFSTreeRoot(ParentIndex) != GetBFSTreeRoot(Index)) && (ParentIndex != Index) && (mNodes[Index].BFSParent != ParentIndex)
                    && (mNodes[ParentIndex].BFSParent != Index)&& (NumberOfCycleNodes()+ mNodes[ParentIndex].BFSDistance+ mNodes[Index].BFSDistance< mMaxNodesInCycles)
                    &&(!(mNodes[ParentIndex].BFSInCycle && mNodes[Index].BFSInCycle));
                    */
                
        }

        public CycleStepResult StepEnumerateCycle()
        {

            if (StepEnumerateCycleStart() == CycleStepResult.Failed)
                return CycleStepResult.Failed;
            CycleStepResult NodeState;

            foreach (int u in mEdges[mCurrentTest.Index])
            {

                NodeState = CheckNode(u);
                switch (NodeState) {

                    case CycleStepResult.Continue:
                        continue;
                    default:

                        return NodeState;
                }
               
            }
            mNodes[mCurrentTest.Index].BFSFinished = true;
            mCurrentTest = null;
//            mCurrentTest.Index = -1;
            return CycleStepResult.Failed;

        }

        public CycleStepResult StepEnumerateCycleRandom()
        {

            if (StepEnumerateCycleStart() == CycleStepResult.Failed)
                return CycleStepResult.Failed;
            CycleStepResult NodeState;
//            if (mEdges[mCurrentTest.Index].Count == 0)
                if (mCurrentNeighbors.Count == 0)

                {
                    mNodes[mCurrentTest.Index].BFSFinished = true;
                mCurrentTest = null;
//                mCurrentTest.Index = -1;
                return CycleStepResult.Failed;
            }
//            int SelectedNodeIndex = mGenerator.Next()% mEdges[mCurrentTest.Index].Count;
//            int SelectedNode = (mEdges[mCurrentTest.Index])[SelectedNodeIndex];


            int SelectedNodeIndex = mGenerator.Next() % mCurrentNeighbors.Count;
            int SelectedNode = (mCurrentNeighbors)[SelectedNodeIndex];


            //foreach (int u in mEdges[mCurrentTest.Index])
            //{

            NodeState = CheckNode(SelectedNode);
            switch(NodeState) {
                case CycleStepResult.Continue:
//                    RemoveEdge(SelectedNode, mCurrentTest.Index);

                 //   mCurrentNeighbors.Remove(SelectedNode);
                    mCurrentNeighbors.RemoveAt(SelectedNodeIndex);

                    break;
                default:
                    return NodeState;
            }

            return CycleStepResult.Failed; 

//            }
//            mBFSFinished[mCurrentTest.Index] = true;
//            mCurrentTest = null;
//            return CycleStepResult.Failed;

        }



        void AddPotetnialRedo()
        {
/*            foreach (int tNewCycleElement in mBFSFoundEars.Ears[mBFSFoundEars.Ea.Count - 1])
            {

                //     mQueue.Enqueue(new BFSNode(tNewCycleElement, mBFSCounter[tNewCycleElement], mBFSDistance[tNewCycleElement], mBFSParent[tNewCycleElement]));
                mNodes[tNewCycleElement].BFSDistance = 0;
                RedoLeafs(tNewCycleElement);
            }
            */


            foreach (int tNewCycleElement in mBFSFoundEars.Ears[mBFSFoundEars.Ears.Count - 1].Nodes)
            {

    //     mQueue.Enqueue(new BFSNode(tNewCycleElement, mBFSCounter[tNewCycleElement], mBFSDistance[tNewCycleElement], mBFSParent[tNewCycleElement]));
                    mNodes[tNewCycleElement].BFSDistance = 0;
                    RedoLeafs(tNewCycleElement);
            }

}
void RedoLeafs(int index) {

if (!mNodes[index].BFSAvailable)
    return;
if(mNodes[index].BFSDescendants!= null) {
                foreach (int desc in mNodes[index].BFSDescendants)
                {
                    if ((!mNodes[desc].BFSInCycle))
                    {
                        if (mNodes[desc].BFSFinished)
                        {
                            mNodes[desc].BFSFinished = false;
                            mNodes[desc].BFSDistance = mNodes[index].BFSDistance + 1;
                            mQueue.Enqueue(new BFSNode(desc, mNodes[desc].BFSCounter, mNodes[desc].BFSDistance, mNodes[desc].BFSParent));
                            RedoLeafs(desc);
                        }
                        else
                        {

                            if (mNodes[desc].BFSDistance > mNodes[index].BFSDistance + 1)
                            {
                                mNodes[desc].BFSDistance = mNodes[index].BFSDistance + 1;
                                RedoLeafs(desc);
                            }
                        }
                    }
                }
}

}

public void Copy( BFSGraph A) {



}

public void UpdateRootFirstCycle() {

foreach (int des in mNodes[mRoot].BFSDescendants) {

    if (!mNodes[des].BFSInCycle) {

        mNodes[des].BFSTreeRootParent = mRoot;
    }
}
}

bool IsFreeRootParent(int index) {

return ((GetBFSTreeRoot(index) == index) && (GetBFSParent(index) == mRoot) && (GetBFSTreeRoot(mRoot) == -1));
}
public void GetHalfCycle(int Index, List<int> HalfCylcle) {

HalfCylcle.Add(Index);

bool next = GetBFSTreeRoot(Index) != Index;

//            mBFSTreeRootParent[Index] = Index;

if ((next) && (mNodes[Index].BFSParent != -1)) {
//    HalfCylcle.Add((mNodes[Index].BFSParent));
    GetHalfCycle(mNodes[Index].BFSParent, HalfCylcle);
}
else
{

    if (IsFreeRootParent(Index))
        GetHalfCycle(mRoot, HalfCylcle); ;
}

}

 public void AddEar(int Node, int Start, int End) {
            List<int> nCycle = new List<int>();
            mUpdateNumberOfNodesInCylce = true;
            nCycle.Add(Start);
            nCycle.Add(Node);
            nCycle.Add(End);
                
            mBFSFoundEars.Add(nCycle, this);
        }

        public void AddEar(int u, int v)
{
List<int> nCycle = new List<int>();
List<int> tHalfCycle1 = new List<int>();
List<int> tHalfCycle2 = new List<int>();

            mUpdateNumberOfNodesInCylce = true;
            GetHalfCycle(u, tHalfCycle1);
GetHalfCycle(v, tHalfCycle2);
tHalfCycle1.Reverse();

foreach (int el in tHalfCycle1) {
    nCycle.Add(el);
}

foreach (int el in tHalfCycle2)
{
                
                nCycle.Add(el);
}

mBFSFoundEars.Add(nCycle, this);

}


public void UpdateInsideCycle(int Index, int iEarIndex) {


// NEEDS FIXING
bool next = GetBFSTreeRoot(Index) != Index;
bool RootParent = ((GetBFSTreeRoot(Index) == Index) && (GetBFSParent(Index) == mRoot) && (GetBFSTreeRoot(mRoot) == -1));
mNodes[Index].BFSTreeRootParent = Index;
mNodes[Index].BFSInCycle = true;
mNodes[Index].BFSDistance = 0;
if(mNodes[Index].EarIndex == -1)
            mNodes[Index].EarIndex = iEarIndex;

if ((next) && (mNodes[Index].BFSParent!= -1))
    UpdateInsideCycle(mNodes[Index].BFSParent, iEarIndex);
else {

    if(RootParent)
        UpdateInsideCycle(mRoot, iEarIndex);
}


}





}
}
