﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HamiltoninaGSD
{
    public class ABSBipartitioning
    {
        ABSGraph                mGraph;
        int                     mNumberOfSubgraphs;
        int[]                   mRootNodes;
        int[]                   mSubgraphsWeights;
        int                     mRandomSeed;
        BFSGraph[]              mSubGraphs;
        Random                  mGenerator;
        SubgraphNode[]          mSubgraphNodes;
        List<int>[]             mSubgraphNeighbors;
        List<int>               mActiveSubgraphs;
        int                     mMaxNumberOfNodesInSubgraph;
        public float            mExceptCoef;

        

        public bool AllSubgraphsNonEmpty() {

            int counter = 0;
            for (int i = 0; i < mNumberOfSubgraphs; i++) {

                if (mSubGraphs[i].NumberOfCycleNodes() > 0)
                    counter++;
            }

            return counter == mNumberOfSubgraphs;
        } 

        public void SetMaxNumberOFNodesInSubgraphs(int iMaxNumberOfInSubgraphs) {
            mMaxNumberOfNodesInSubgraph = iMaxNumberOfInSubgraphs;

            for (int i = 0; i < mNumberOfSubgraphs; i++)
                mSubGraphs[i].MaxNodesInCycles = mMaxNumberOfNodesInSubgraph;

        }
        public List<int>[] GetSubgraphNeighbors() { return mSubgraphNeighbors; }

        public void CopySubgraph(ABSBipartitioning A, int index)
        {

           

            
            if (mSubGraphs[index] != null)
            {
                mSubGraphs[index].CopyData(A.GetSubgraph(index));
            }
            else {
                mSubGraphs[index] = new BFSGraph(A.GetSubgraph(index));
            }

//            if (A.mSubgraphNeighbors != null)
//            {
                mSubgraphNeighbors = new List<int>[mNumberOfSubgraphs + 1];

                for (int i = 0; i < mNumberOfSubgraphs + 1; i++)
                {

                    mSubgraphNeighbors[i] = A.mSubgraphNeighbors[i].ToList();

                }
//            }
        }
        public void GetFreeNodeCluster(int Seed, List<int> Cluster, List<int> SubgraphNeighbors) {

             Cluster.Add(Seed);
            foreach (int n in mGraph.GetConnections(Seed)) {
                if ((mSubgraphNodes[n].SubgraphId == -1))
                {
                    if ((!Cluster.Contains(n)))
                        GetFreeNodeCluster(n, Cluster, SubgraphNeighbors);
                }
                else
                {

                    if (mSubgraphNodes[n].SubgraphId != -1)
                    {
                        if (!SubgraphNeighbors.Contains(mSubgraphNodes[n].SubgraphId))
                            SubgraphNeighbors.Add(mSubgraphNodes[n].SubgraphId);

                    }
                }
            }
        }

        public void AddAllSubgraphConnectionsForCluster() {

            List<int> FreeNodes = GetFreeNodes();
            List<int> Cluster = new List<int>();
            List<int> SubgraphNeighbors = new List<int>();

            while (FreeNodes.Count > 0) {

                Cluster.Clear();
                SubgraphNeighbors.Clear();
                GetFreeNodeCluster(FreeNodes[0], Cluster, SubgraphNeighbors);

                foreach (int ce in Cluster) {

                    FreeNodes.Remove(ce);
                }
                foreach (int s1 in SubgraphNeighbors)
                    foreach (int s2 in SubgraphNeighbors) {

                        if (!mSubgraphNeighbors[s1].Contains(s2)) {
                            mSubgraphNeighbors[s1].Add(s2);
                        }

                        if (!mSubgraphNeighbors[s2].Contains(s1))
                        {
                            mSubgraphNeighbors[s2].Add(s1);
                        }

                    }

            }
        }


        public void CalculateSubgraphNeighbors1(ABSGraph OriginalGraph ) {


                    bool[] UsedNodes = Enumerable.Repeat<bool>(false, mGraph.NumberOfVertices).ToArray() ;

                    List<int> Neighbors ;
                    int       cSubGraphId;
                    int       nSubGraphId;
                    List<int> eNeighbors;

                    int n;



                    mSubgraphNeighbors = new List<int>[mNumberOfSubgraphs + 1];
                    for (int i = 0; i < mNumberOfSubgraphs+1; i++)
                        mSubgraphNeighbors[i] = new List<int>();

                    CollectSubgraphNodes();

                    for (int i = 0; i < mGraph.NumberOfVertices; i++) {
                        Neighbors = new List<int>(OriginalGraph.GetConnections(i));

                        for (int ii = 0; ii < Neighbors.Count; ii++)
                        {


                                n = Neighbors[ii];
                                //                foreach (int n in Neighbors.ToList()) {

                                UsedNodes[Neighbors[ii]] = true;

                                cSubGraphId = mSubgraphNodes[i].SubgraphId;

                                if (cSubGraphId == -1)
                                    cSubGraphId = NumberOfSubgraphs;

                                nSubGraphId = mSubgraphNodes[n].SubgraphId;

                                if (nSubGraphId == -1)
                                {
                                    nSubGraphId = NumberOfSubgraphs;
/*                                    eNeighbors = OriginalGraph.GetConnections(n);
                                    foreach (int en in eNeighbors)
                                    {
                                        if (mSubgraphNodes[en].SubgraphId == -1)
                                        {
                                            if (!Neighbors.Contains(en))
                                                Neighbors.Add(en);
                                        }
                                    }

                                */
                                }



/*                                if (!mSubgraphNeighbors[cSubGraphId].Contains(nSubGraphId) && (nSubGraphId < NumberOfSubgraphs) &&
                                    (nSubGraphId != cSubGraphId))
                                {
                                    mSubgraphNeighbors[cSubGraphId].Add(nSubGraphId);
        //                            mSubgraphNeighbors[nSubGraphId].Add(cSubGraphId);
                                }
                     */         
                                if (!mSubgraphNeighbors[nSubGraphId].Contains(cSubGraphId) && (nSubGraphId < NumberOfSubgraphs) &&
                                    (nSubGraphId != cSubGraphId))
                                {
                                    mSubgraphNeighbors[nSubGraphId].Add(cSubGraphId);
                                }
                                


    }
            }


            }


        public void CalculateSubgraphNeighbors(ABSGraph OriginalGraph)
        {


            bool[] UsedNodes = Enumerable.Repeat<bool>(false, mGraph.NumberOfVertices).ToArray();

            bool[,] cNeighbors = new Boolean[NumberOfSubgraphs, NumberOfSubgraphs+1] ;
            List<int> Neighbors;
            int cSubGraphId;
            int nSubGraphId;
            List<int> eNeighbors;

            //      int n;

            for (int i = 0; i < mNumberOfSubgraphs; i++)
                for (int j = 0; j < mNumberOfSubgraphs + 1; j++)
                    cNeighbors[i, j] = false;



                    mSubgraphNeighbors = new List<int>[mNumberOfSubgraphs + 1];
            for (int i = 0; i < mNumberOfSubgraphs + 1; i++)
                mSubgraphNeighbors[i] = new List<int>();

            CollectSubgraphNodes();

            for (int i = 0; i < mGraph.NumberOfVertices; i++)
            {
                foreach(int n in OriginalGraph.GetConnections(i))
                {
                    UsedNodes[n] = true;
                    cSubGraphId = mSubgraphNodes[i].SubgraphId;

                    if (cSubGraphId == -1)
                        cSubGraphId = NumberOfSubgraphs;

                    nSubGraphId = mSubgraphNodes[n].SubgraphId;

                    if (nSubGraphId == -1)
                    {
                        nSubGraphId = NumberOfSubgraphs;
                    }


               /*   
                    if (!mSubgraphNeighbors[cSubGraphId].Contains(nSubGraphId) && (nSubGraphId < NumberOfSubgraphs) &&
                        (nSubGraphId != cSubGraphId))
                    {
                        mSubgraphNeighbors[cSubGraphId].Add(nSubGraphId);
                    }
                    /**/
                   
                    if ((nSubGraphId < NumberOfSubgraphs) &&
                     (nSubGraphId != cSubGraphId))
                    {
                        cNeighbors[nSubGraphId, cSubGraphId] = true;
//                        mSubgraphNeighbors[cSubGraphId].Add(nSubGraphId);
                    }

                    /**/

                    //                    if (!mSubgraphNeighbors[cSubGraphId].Contains(nSubGraphId) && (nSubGraphId < NumberOfSubgraphs) &&
                    /*                    if ((nSubGraphId > cSubGraphId)  && (nSubGraphId < NumberOfSubgraphs))
                                        {
                                            mSubgraphNeighbors[cSubGraphId].Add(nSubGraphId);
                                            mSubgraphNeighbors[nSubGraphId].Add(cSubGraphId);
                                         }
                                         */

                }
            }
                  for (int i = 0; i < mNumberOfSubgraphs+1; i++)
                       for (int j = 0; j < mNumberOfSubgraphs; j++) {

                           if (cNeighbors[j, i])
                               mSubgraphNeighbors[i].Add(j);
                       }

            for (int i = 0; i < mNumberOfSubgraphs + 1; i++)
                shuffle(mSubgraphNeighbors[i]);
            /*                  */
            //                    cNeighbors[i, j] = false;


        }


        public ABSGraph GetGraph() { return mGraph; }
        public int NumberOfSubgraphs
        {
            get { return mNumberOfSubgraphs; }
            set { mNumberOfSubgraphs = value; }

        }
        public SubgraphNode GetSubgraphNode(int index) {
            return mSubgraphNodes[index];
        }


        public void RemoveNodeFromCycles(int SelectNode) {

            if (SelectNode != -1)
                if (mSubgraphNodes[SelectNode].SubgraphId != -1) {
                    mSubGraphs[mSubgraphNodes[SelectNode].SubgraphId].RemoveNodeFromCycles(SelectNode);
                }
        }
       /*
        public List<int[]> CheckSimpleAdd(int NodeId, ABSGraph OriginalGraph, out int start, out int end)
        {

            List<int[]> Result = new List<int[]>();
            List<int> Neighbors;

            end = -1;
            start = -1;
            


            Neighbors = OriginalGraph.GetConnections(NodeId);
            foreach (int u in Neighbors)
                foreach (int v in Neighbors)
                {
                    if ((mSubgraphNodes[u].SubgraphId == mSubgraphNodes[v].SubgraphId) && (u != v) && (mSubgraphNodes[v].SubgraphId != -1))
                    {
                        int[] add = { u, v, mSubgraphNodes[u].SubgraphId };
                        start = v;
                        end = u;
//                        if(!Result.Contains(mSubgraphNodes[u].SubgraphId) )
                                Result.Add(add);
                    }
                }

            return Result;

        }
*/

        public List<int[]> CheckSimpleAdd(int NodeId, ABSGraph OriginalGraph, out int start, out int end)
        {

            List<int[]> Result = new List<int[]>();
            List<int>[] PotEars = new List<int>[NumberOfSubgraphs];
            List<int> Neighbors = new List<int>();
            List<int> SubNeighbors = new List<int>();

            end = -1;
            start = -1;



            Neighbors = OriginalGraph.GetConnections(NodeId);
            foreach (int u in Neighbors)
            {

                if (mSubgraphNodes[u].SubgraphId != -1)
                {
                    if (PotEars[mSubgraphNodes[u].SubgraphId] == null)
                        PotEars[mSubgraphNodes[u].SubgraphId] = new List<int>();

                    if (!PotEars[mSubgraphNodes[u].SubgraphId].Contains(u))
                        PotEars[mSubgraphNodes[u].SubgraphId].Add(u);

                    if (!SubNeighbors.Contains(mSubgraphNodes[u].SubgraphId))
                        SubNeighbors.Add(mSubgraphNodes[u].SubgraphId);
                }


                shuffle(SubNeighbors);
                foreach (int ss in SubNeighbors)
                {
                    if (PotEars[ss].Count > 1)
                    {
                        shuffle(PotEars[ss]);
                        int[] add = { (PotEars[ss])[0], (PotEars[ss])[1], mSubgraphNodes[(PotEars[ss])[1]].SubgraphId };
                        Result.Add(add);
                        break;
                    }
                }
            }
            return Result;

        }


        public bool SimpleImprove(ABSGraph OriginalGraph) {
            bool result = false;
            int start, end;
            List<int[]>  impSubgraph;

            CollectSubgraphNodes();
            List<int> freeNodes = GetFreeNodes();
           shuffle(freeNodes);
 

            foreach (int tn in freeNodes) {
                impSubgraph = CheckSimpleAdd(tn, OriginalGraph, out start, out end);
                shuffle(impSubgraph);
                foreach (int[] ss in impSubgraph) {


                  if (mSubGraphs[ss[2]].CanAdd(1)) {


                        mSubGraphs[ss[2]].SetNodeBFSInCycle(tn, true);
                        mSubGraphs[ss[2]].SetNodeBFSTreeRootParent(tn, tn);
                        mSubGraphs[ss[2]].SetNodeBFSDistance(tn, 0);


                   //     mSubGraphs[ss[2]].SetNodeBFSEarIndex(tn, int vEarIndex);


//        mSubGraphs[ss[2]].GetNode(tn).BFSInCycle = true;
                         mSubgraphNodes[tn].SubgraphId = ss[2];

//                         mSubGraphs[ss[2]].GetNode(tn).BFSTreeRootParent = tn;
//                         mSubGraphs[ss[2]].GetNode(tn).BFSInCycle = true;
//                         mSubGraphs[ss[2]].GetNode(tn).BFSDistance = 0;


                         mSubGraphs[ss[2]].AddEar(tn, ss[0], ss[1]);

                        if (mSubGraphs[ss[2]].GetNode(tn).EarIndex == -1)
                            //                              mSubGraphs[ss[2]].GetNode(tn).EarIndex = mSubGraphs[ss[2]].GetEars().Ears.Last().Id;
                            mSubGraphs[ss[2]].SetNodeBFSEarIndex(tn, mSubGraphs[ss[2]].GetEars().Ears.Last().Id);

                        result = true;
                    break;
                }
                  }
            }

                return result;
                
        }

        public  void shuffle(List<int> list)
        {
//            Random rng = new Random();   // i.e., java.util.Random.
            int n = list.Count;        // The number of items left to shuffle (loop invariant).
            while (n > 1)
            {
                int k = mGenerator.Next(n);  // 0 <= k < n.
                n--;                     // n is now the last pertinent index;
                int temp = list[n];     // swap array[n] with array[k] (does nothing if k == n).
                list[n] = list[k];
                list[k] = temp;
            }
        }
        public void shuffle(List<int[]> list)
        {
//            Random rng = new Random();   // i.e., java.util.Random.
            int n = list.Count;        // The number of items left to shuffle (loop invariant).
            while (n > 1)
            {
                int k = mGenerator.Next(n);  // 0 <= k < n.
                n--;                     // n is now the last pertinent index;
                int[] temp = list[n];     // swap array[n] with array[k] (does nothing if k == n).
                list[n] = list[k];
                list[k] = temp;
            }
        }

        /*
        public bool FreeAddInCycle(int NodeIndex, ABSGraph OriginalGraph) {

            bool result = false;
            int start, end;
            List<int[]> impSubgraph;

            impSubgraph = CheckSimpleAdd(NodeIndex, OriginalGraph, out start, out end);
                
            foreach (int ss in impSubgraph)
            {
                mSubGraphs[impSubgraph[0]].GetNode(NodeIndex).BFSInCycle = true;
                         mSubgraphNodes[NodeIndex].SubgraphId = impSubgraph[0];

                         mSubGraphs[impSubgraph[0]].GetNode(NodeIndex).BFSTreeRootParent = NodeIndex;
                         mSubGraphs[impSubgraph[0]].GetNode(NodeIndex).BFSInCycle = true;
                         mSubGraphs[impSubgraph[0]].GetNode(NodeIndex).BFSDistance = 0;


                         mSubGraphs[impSubgraph[0]].AddEar(NodeIndex, start, end);

                        if (mSubGraphs[impSubgraph[0]].GetNode(NodeIndex).EarIndex == -1)
                                mSubGraphs[impSubgraph[0]].GetNode(NodeIndex).EarIndex = mSubGraphs[impSubgraph[0]].GetEars().Ears.Last().Id;
                    result = true;
                    break;
                }


         return result;
        }
        */
        public void CollectSubgraphNodes() {


            bool cIsInCycle;
            mSubgraphNodes = new SubgraphNode[mGraph.NumberOfVertices];
            for (int i = 0; i < mGraph.NumberOfVertices; i++) {
                cIsInCycle = false;

                for (int j = 0; j < mNumberOfSubgraphs; j++) {

                    if (mSubGraphs[j].IsInCycle(i)) {
                        cIsInCycle = true;
                        mSubgraphNodes[i] = new SubgraphNode(i, j, mSubGraphs[j].GetNode(i).EarIndex);
                        break;
                    }
                }
                if (!cIsInCycle)
                    mSubgraphNodes[i] = new SubgraphNode(i);

            }
        }
        public void FinishAll() {

            for (int i = 0; i < mNumberOfSubgraphs; i++)
                mSubGraphs[i].Finished = true;
        }
        public void GetNodesInCycleStateAll( bool[] oInCycle, List<int>[]  oNodesInSubgraph = null) {
            int k = 7;
            for (int i = 0; i < mGraph.NumberOfVertices; i++) {

                oInCycle[i] = false;
            }


            for (int i = 0; i < NumberOfSubgraphs; i++)
            {
                if (oNodesInSubgraph != null)
                    oNodesInSubgraph[i].Clear();

                for (int j = 0; j < mGraph.NumberOfVertices; j++)
                {
//                    if(oInCycle[j] && mSubGraphs[i].IsInCycle(j))
//                            k = 7;   
                    oInCycle[j] |= mSubGraphs[i].IsInCycle(j);

                    if (oNodesInSubgraph != null)
                    {
                        if (mSubGraphs[i].IsInCycle(j))
                            oNodesInSubgraph[i].Add(j);
                    }

                }
            }
                
        }

        public void Copy(ABSBipartitioning A, bool All = true) {

            mGraph = new ABSGraph(A.mGraph);
            mNumberOfSubgraphs = A.mNumberOfSubgraphs;
            Array.Copy(A.mRootNodes, mRootNodes, mNumberOfSubgraphs);
            
            if (A.mSubgraphsWeights != null)
                Array.Copy(A.mSubgraphsWeights, mSubgraphsWeights, mNumberOfSubgraphs);
            else
            {
                mSubgraphsWeights = null;
            }



            mRandomSeed = A.mRandomSeed;

            //mSubGraphs = new BFSGraph[mNumberOfSubgraphs];
            for (int i = 0; i < mNumberOfSubgraphs; i++)
            {
                mSubGraphs[i].CopyData(A.GetSubgraph(i));
            }
            mGenerator = new Random(mRandomSeed);

            mSubgraphNeighbors = new List<int>[mNumberOfSubgraphs + 1];

            for (int i = 0; i < mNumberOfSubgraphs + 1; i++)
            {

                mSubgraphNeighbors[i] = A.mSubgraphNeighbors[i].ToList();
            }
            mMaxNumberOfNodesInSubgraph = A.mMaxNumberOfNodesInSubgraph;


        }
        public ABSBipartitioning(ABSBipartitioning A, bool All = true) {

            mGraph = new ABSGraph(A.mGraph);
            mNumberOfSubgraphs = A.mNumberOfSubgraphs;
            mRootNodes = new int[mNumberOfSubgraphs];
         
            Array.Copy(A.mRootNodes, mRootNodes, mNumberOfSubgraphs);

            mSubgraphsWeights = new int[mNumberOfSubgraphs];
            if(A.mSubgraphsWeights!= null)
                Array.Copy(A.mSubgraphsWeights, mSubgraphsWeights, mNumberOfSubgraphs);
            else{
                mSubgraphsWeights = null;
            }



            mRandomSeed = A.mRandomSeed;

            mSubGraphs = new BFSGraph[mNumberOfSubgraphs];
            for (int i = 0; i < mNumberOfSubgraphs; i++) {
                mSubGraphs[i] = new BFSGraph(A.GetSubgraph(i), All);
            }
            mGenerator = new Random(mRandomSeed);


//            if (mSubgraphNeighbors != null)
//            {
                mSubgraphNeighbors = new List<int>[mNumberOfSubgraphs + 1];
                for (int i = 0; i < mNumberOfSubgraphs + 1; i++)
                {
                    mSubgraphNeighbors[i] = A.mSubgraphNeighbors[i].ToList();
                }
//            }
            mMaxNumberOfNodesInSubgraph = A.mMaxNumberOfNodesInSubgraph;
        }
        public void SetAllGenerators() {

            for (int i = 0; i < mNumberOfSubgraphs; i++) {
                mSubGraphs[i].Generator = mGenerator;
            }
        }



        public void SetGenerator(Random iGenerator) { mGenerator = iGenerator; }

        void SetSubgraphRoots() {

            for (int i = 0; i < mNumberOfSubgraphs; i++) {

                mSubGraphs[i].Root = mRootNodes[i];
            }
        }

        public void SavePartition(System.IO.StreamWriter textFile)
        {

            List<int> cSubNodes;  
            for (int i = 0; i < mNumberOfSubgraphs; i++) {
                
                textFile.WriteLine("mRootNodes Node");
                textFile.WriteLine(mRootNodes[i]);


                    cSubNodes = mSubGraphs[i].GetNodesInCycles();
                    foreach(int s in cSubNodes)
                        textFile.WriteLine(s);

            }
        }

        public void SavePartition(string FileName) {

            System.IO.StreamWriter textFile = new System.IO.StreamWriter(FileName);
            SavePartition(textFile);
            textFile.Close();


        }
        public void DeleteTreeNodeInAllSubgraphs(int Node, int ExcludedSubgraph=-1, List<int> Active = null) {

            if (Active == null)
            {

                for (int i = 0; i < NumberOfSubgraphs; i++)
                {

                    if (ExcludedSubgraph != i)
                    {

                        mSubGraphs[i].DeleteTreeNode(Node, true, false);

                    }
                }
            }
            else {

                foreach (int i in Active) {

                    if (ExcludedSubgraph != i)
                    {

                        mSubGraphs[i].DeleteTreeNode(Node, true, false);

                    }
                }
            }
        }

        public void DeleteTreeNodeInSubgraphList(int Node, List<int> SubgraphList, int ExcludedSubgraph = -1)
        {

            foreach (int i in SubgraphList)
            {

                if (ExcludedSubgraph != i)
                {

                    mSubGraphs[i].DeleteTreeNode(Node, true, false);

                }
            }
        }


        void RandomRoots()
        {
            mRootNodes = new int[NumberOfSubgraphs];

            for (int i = 0; i < mNumberOfSubgraphs; i++)
            {

                mRootNodes[i] = mGenerator.Next()% mGraph.NumberOfVertices;
            }
        }
        /*
        void SetRoots(int[] Roots)
        {
            mRootNodes = new int[NumberOfSubgraphs];

            for (int i = 0; i < mNumberOfSubgraphs; i++)
            {

                mRootNodes[i] = Roots[i];
            }
        }
        */

        public void Start() {


            for (int i = 0; i < mNumberOfSubgraphs; i++)
            {

                mSubGraphs[i].StartEnumerate();
                mSubGraphs[i].ExceptCoef = mExceptCoef;
                
            }
            for (int i = 0; i < mNumberOfSubgraphs; i++)
                DeleteTreeNodeInAllSubgraphs(mSubGraphs[i].Root, i);
            SetAllGenerators();
        }

        
        public void StartParitial(int index, List<int> SubgraphList = null)
        {



                mSubGraphs[index].StartEnumerate();
            if (SubgraphList == null)
            {
                for (int i = 0; i < mNumberOfSubgraphs; i++)
                    DeleteTreeNodeInAllSubgraphs(mSubGraphs[i].Root, i);
            }
            else {
                for (int i = 0; i < mNumberOfSubgraphs; i++)
                    DeleteTreeNodeInSubgraphList(mSubGraphs[i].Root,SubgraphList, i);

            }
            SetAllGenerators();

        }

        public ABSBipartitioning() {

            mNumberOfSubgraphs = 2;
              
            mSubGraphs = new BFSGraph[2];

        }

        public ABSBipartitioning(int iNumberOfSubgraphs)
        {

            mNumberOfSubgraphs = iNumberOfSubgraphs;

            mSubGraphs = new BFSGraph[mNumberOfSubgraphs];

        }


        public ABSBipartitioning(int iNumberOfSubgraphs, ABSGraph iGraph)
        {

            mNumberOfSubgraphs = iNumberOfSubgraphs;
            mGraph = new BFSGraph(iGraph);
            mSubGraphs = new BFSGraph[mNumberOfSubgraphs];
//            CreateAllSubgraphs();
        }

        public void SetRoots(int[]  nRoots) {
            mRootNodes = new int[mNumberOfSubgraphs];
            for (int i = 0; i < mNumberOfSubgraphs; i++) {
                mRootNodes[i] = nRoots[i];
            }

        }
        public void AllocRoots() {

            mRootNodes = new int[NumberOfSubgraphs];
        }
        public void CreateAllSubgraphs(bool NewRoot=false) {

            
            for (int i = 0; i < mNumberOfSubgraphs; i++) {
                mSubGraphs[i] = new BFSGraph(mGraph);
//                mSubGraphs[i].InitBFSEnumeration();
                mSubGraphs[i].AllocBFSEnumeration();
                mSubGraphs[i].Root = mRootNodes[i];
                mSubGraphs[i].MaxNodesInCycles = mMaxNumberOfNodesInSubgraph;
            }

//            RandomRoots();
//            SetSubgraphRoots();
        }

        public void RestartSubgraph(int index)
        {

//                mSubGraphs[index] = new BFSGraph(mGraph);


            mSubGraphs[index].Restart();
//                mSubGraphs[index].InitBFSEnumeration();
                mSubGraphs[index].Root = mRootNodes[index];
           

            //            RandomRoots();
            //            SetSubgraphRoots();
        }



        public void InitSubgraphs(){


        }


        public void SetGraph(ABSGraph nGraph) {

            mGraph = nGraph;
            
        }
        public BFSGraph GetSubgraph(int index)
        {
            return mSubGraphs[index];
            
        }


        public void InitPartitions(int nNumberOfParitions, int[] nRoots, int[] nWeights) {

        }

        List<int> GetFreeNodes() {

            List<int> result = new List<int>();
            for (int i = 0; i < mGraph.NumberOfVertices; i++) {
                if (mSubgraphNodes[i].SubgraphId == -1)
                    result.Add(i);
            }
            return result;
        }

    }
}
