﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HamiltoninaGSD
{
    public class BFSNodeContainer
    {
        protected int mNumberOfNodes;

        protected int[]       mDistance;
        protected int[]       mBFSParent;
        protected int[]       mBFSCounter;
        protected int[]       mBFSTreeRoot;
        protected int[]       mBFSTreeRootParent;
        protected bool[]      mFinished;
        protected bool[]      mAvailable;
        protected bool[]      mInCycle;
        protected int[]       mEarIndex;
        protected List<int>[] mBFSDescendants;


        static protected int[] sDistance;
        static protected int[] sBFSParent;
        static protected int[] sBFSCounter;
        static protected int[] sBFSTreeRoot;
        static protected int[] sBFSTreeRootParent;
        static protected bool[] sFinished;
        static protected bool[] sAvailable;
        static protected bool[] sInCycle;
        static protected int[] sEarIndex;
        static protected List<int>[] sBFSDescendants;

        static public void InitStatic(int cNumberOfNodes) {

            sDistance = Enumerable.Repeat<int>(-1, cNumberOfNodes).ToArray();
            sBFSParent = Enumerable.Repeat<int>(-1, cNumberOfNodes).ToArray();
            sBFSCounter = Enumerable.Repeat<int>(-1, cNumberOfNodes).ToArray();
            sBFSTreeRoot = Enumerable.Repeat<int>(-1, cNumberOfNodes).ToArray();
            sBFSTreeRootParent = Enumerable.Repeat<int>(-1, cNumberOfNodes).ToArray();
            sEarIndex = Enumerable.Repeat<int>(-1, cNumberOfNodes).ToArray();


            sInCycle = Enumerable.Repeat<bool>(false, cNumberOfNodes).ToArray();
            sFinished = Enumerable.Repeat<bool>(false, cNumberOfNodes).ToArray();
            sAvailable = Enumerable.Repeat<bool>(true, cNumberOfNodes).ToArray();

            sBFSDescendants = Enumerable.Repeat<List<int>>(null, cNumberOfNodes).ToArray();

        }


        


        public BFSNode GetNode(int index) {
            BFSNode result;
            result = new BFSNode();

            result.BFSAvailable         = mAvailable[index];
            result.BFSParent            = mBFSParent[index];
            result.BFSCounter           = mBFSCounter[index];
            result.BFSTreeRoot          = mBFSTreeRoot[index];
            result.BFSTreeRootParent    = mBFSTreeRootParent[index];
            result.BFSFinished          = mFinished[index];
            result.EarIndex             = mEarIndex[index];
            result.BFSInCycle          =  mInCycle[index];
            result.BFSDescendants = mBFSDescendants[index];

            return result;


        }
        public void AddDescendant(int index, int Desc) {
            mBFSDescendants[index].Add(Desc);
        }
        public void RemoveDescendant(int index, int Desc)
        {
            mBFSDescendants[index].Remove(Desc);
        }
        public void ClearDescendants(int index)
        {
            mBFSDescendants[index].Clear();
        }
        public void AllocDescendants(int index)
        {
            mBFSDescendants[index] = new List<int>();
        }

        public List<int> GetDescendants(int index)
        {
            return mBFSDescendants[index];
        }

        public void SetDistance(int index, int Value) {
            mDistance[index] = Value;
        }
        public int GetDistance(int index)
        {
            return mDistance[index];
        }

        public void SetBFSParent(int index, int Value)
        {
            mBFSParent[index] = Value;
        }
        public int GetBFSParent(int index)
        {
            return mBFSParent[index];
        }


        public void SetBFSCounter(int index, int Value)
        {
            mBFSCounter[index] = Value;
        }
        public int GetBFSCounter(int index)
        {
            return mBFSCounter[index];
        }

        public void SetBFSTreeRoot(int index, int Value)
        {
            mBFSTreeRoot[index] = Value;
        }
        public int GetBFSTreeRoot(int index)
        {
            return mBFSTreeRoot[index];
        }


        public void SetBFSTreeRootParent(int index, int Value)
        {
            mBFSTreeRootParent[index] = Value;
        }
        public int GetBFSTreeRootParent(int index)
        {
            return mBFSTreeRootParent[index];
        }


        public void SetEarIndex(int index, int Value)
        {
            mEarIndex[index] = Value;
        }
        public int GetEarIndex(int index)
        {
            return mEarIndex[index];
        }


        public void SetAvailable(int index, bool Value)
        {
            mAvailable[index] = Value;
        }
        public bool GetAvailable(int index)
        {
            return mAvailable[index];
        }


        public void SetInCycle(int index, bool Value)
        {
            mInCycle[index] = Value;
        }
        public bool GetInCycle(int index)
        {
            return mInCycle[index];
        }

        public void SetFinished(int index, bool Value)
        {
            mFinished[index] = Value;
        }
        public bool GetFinished(int index)
        {
            return mFinished[index];
        }


        public BFSNodeContainer(int nNumberOFNodes) {

            mNumberOfNodes = nNumberOFNodes;
            Alloc();
            NewInit();
                
        }
        public void NewInit() {

//            if(mBFSDescendants== null)
//            mBFSDescendants = new List<int>[mNumberOfNodes];


//            mAvailable = Enumerable.Repeat<List<int>>(null, mNumberOfNodes).ToArray();

          /*  mDistance = Enumerable.Repeat<int>(-1, mNumberOfNodes).ToArray();
            mBFSParent   = Enumerable.Repeat<int>(-1, mNumberOfNodes).ToArray();
            mBFSCounter  = Enumerable.Repeat<int>(-1, mNumberOfNodes).ToArray();
            mBFSTreeRoot = Enumerable.Repeat<int>(-1, mNumberOfNodes).ToArray();
            mBFSTreeRootParent = Enumerable.Repeat<int>(-1, mNumberOfNodes).ToArray();
            mEarIndex = Enumerable.Repeat<int>(-1, mNumberOfNodes).ToArray();


            mInCycle   = Enumerable.Repeat<bool>(false, mNumberOfNodes).ToArray();
            mFinished  = Enumerable.Repeat<bool>(false, mNumberOfNodes).ToArray();
            mAvailable = Enumerable.Repeat<bool>(true, mNumberOfNodes).ToArray();
            mBFSDescendants = Enumerable.Repeat<List<int>>(null, mNumberOfNodes).ToArray();
            */
            
            Array.Copy(sAvailable, mAvailable, mNumberOfNodes);
            Array.Copy(sFinished, mFinished, mNumberOfNodes);
            Array.Copy(sInCycle, mInCycle, mNumberOfNodes);


            Array.Copy(sDistance, mDistance, mNumberOfNodes);
            Array.Copy(sBFSParent, mBFSParent, mNumberOfNodes);
            Array.Copy(sBFSCounter, mBFSCounter, mNumberOfNodes);
            Array.Copy(sBFSTreeRoot, mBFSTreeRoot, mNumberOfNodes);
            Array.Copy(sBFSTreeRootParent, mBFSTreeRootParent, mNumberOfNodes);
            Array.Copy(sEarIndex, mEarIndex, mNumberOfNodes);
            Array.Copy(sBFSDescendants, mBFSDescendants, mNumberOfNodes);
            /**/

        }
        public void Alloc() {

            mDistance = new int[mNumberOfNodes];
            mBFSParent = new int[mNumberOfNodes]; 
            mBFSCounter = new int[mNumberOfNodes]; 
            mBFSTreeRoot = new int[mNumberOfNodes];
            mBFSTreeRootParent = new int[mNumberOfNodes];
            mEarIndex = new int[mNumberOfNodes];


            mInCycle = new bool[mNumberOfNodes];
            mFinished = new bool[mNumberOfNodes];
            mAvailable = new bool[mNumberOfNodes];

            mBFSDescendants = new List<int>[mNumberOfNodes];


        }


        public void Copy(BFSNodeContainer A)
        {

            Array.Copy(A.mAvailable, mAvailable, A.mNumberOfNodes);
            Array.Copy(A.mFinished, mFinished, A.mNumberOfNodes);
            Array.Copy(A.mInCycle, mInCycle, A.mNumberOfNodes);


            Array.Copy(A.mDistance, mDistance, A.mNumberOfNodes);
            Array.Copy(A.mBFSParent, mBFSParent, A.mNumberOfNodes);
            Array.Copy(A.mBFSCounter, mBFSCounter, A.mNumberOfNodes);
            Array.Copy(A.mBFSTreeRoot, mBFSTreeRoot, A.mNumberOfNodes);
            Array.Copy(A.mBFSTreeRootParent, mBFSTreeRootParent, A.mNumberOfNodes);
            Array.Copy(A.mEarIndex, mEarIndex, A.mNumberOfNodes);
       
    }
        

}
}
