﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;

using System.IO;

namespace HamiltoninaGSD
{

    public enum CalcType { Random, Neighbor}
    public class BipartitioningProblem
    {
        //ProblemVisualiser       mVisualiser;
        ABSBipartitioning mPartitioning;
        ABSBipartitioning mBestPartitioning;
        Random            mGenerator;
        ABSGraph          mOriginalGraph;
        int               mSeed;
        int               mCurrentSubgraph;
        int               mBestSolution;
        int               mBestIteration;
        long               mBestCalcTime;
        List<int>         mSizes;
        List<long>         mTimes;
        Stopwatch         mStopWatch;
        int               mRegrowSize;
        int               mExpLength;
        float             mExpCoef;



        bool mInitialRoots;
        int[]             mRoots;
        int               mNumberOfGeneratedSolutions;
        int               mMaxNodesInSubgraphs;
        
        CalcType          mCalcType;
        long              mCalcTime;
        string            mFileName;
        float             mExceptCoef;




        public int RegrowSize
        {

            get { return mRegrowSize; }
            set { mRegrowSize = value; }

        }

        public int ExpLength
        {

            get { return mExpLength; }
            set { mExpLength = value; }

        }


        public string GetNameCalc() {

            string result = "";

            switch(mCalcType){
                case CalcType.Neighbor:
                    result = "N";
                    break;
                case CalcType.Random:
                    result = "R";
                    break;

            }
            result += "_EC" + ((int)(mExceptCoef * 100)/10);
            result += "_RG" + RegrowSize;
            result += "_EL" + ExpLength;


            return result;
        }


        public long CalcTime
        {

            get { return mCalcTime; }
            set { mCalcTime = value; }

        }
        public float ExceptCoef {

            get { return mExceptCoef; }
            set { mExceptCoef = value; }
        }
        public CalcType CalcType {

            get { return mCalcType; }
            set { mCalcType = value; }
        }

        public int BestIteration
        {

            get { return mBestIteration; }
            set { mBestIteration = value; }
        }


        public string GenerateFileNameProblems(int nNumberOfNodes, int nNumberOfSubraphs, int seed) {
            string Results = "prob" + "_" + nNumberOfSubraphs + "_" + nNumberOfNodes + "_" + seed;

            return Results;

        }
        public void ClearBest() {

            mBestSolution = 0;
            mBestPartitioning = null;
            mNumberOfGeneratedSolutions = 0;
        }
        public void SaveIntermediateSolutions(string FileName) {

            StreamWriter File = new StreamWriter(FileName+ (((int)(mExceptCoef * 100)) / 10) + ".txt");

            for (int i = 0; i < mSizes.Count; i++) {

                File.WriteLine(mSizes[i] + "   " + mTimes[i]);
            }

            File.Close();
        }
        public void MultipleSolveMatPower(string FileName, int NumberOfRuns, out List<int> Results, out List<int> Opt, out List<int> Iter, out List<long> iTime) {

            Opt = new List<int>();
            Iter = new List<int>();
            iTime = new List<long>();
            Results = new List<int>();

            for (int i = 0; i < NumberOfRuns; i++)
            {


                Load(".\\RW\\" + FileName + ".pro");

                SetAllMaxNodesInCycle();
                //                    tProblem[j].CalcType = CalcType.Neighbor;
                NewGenerator(i);
                Start();
                ClearBest();
                FindSolutionPartial(10000, 1);

                Iter.Add(mBestIteration);
                iTime.Add(mBestCalcTime);
                Results.Add(mBestSolution);
            }


        }
           public void  SolveProblemSet(int nNumberOfNodes, int nNumberOfSubraphs, out List<int> Results, out List<int> Opt, out List<int> Iter, out List<long> iTime, string Folder)
        {


            String tString;
            Results = new List<int>();
            Opt = new List<int>();
            Iter = new List<int>();
            iTime = new List<long>();
            int nNumEdges = 0;
            int Sum = 0;
            int Solution;

            int Value, Value1;
            mCalcTime = 0;

            System.IO.StreamWriter file = new System.IO.StreamWriter("!log.txt");
            file.Close();


            for (int i = 0; i < 40; i++)
            {



                file = new System.IO.StreamWriter("!log.txt", true);

                file.WriteLine(i);

                file.Close();

                mFileName = Folder + GenerateFileNameProblems(nNumberOfNodes, nNumberOfSubraphs, i);
                mFileName = mFileName + ".pro";
                Load(mFileName);

                NumberOfGeneratedSolutions = 0;
                SetAllMaxNodesInCycle();
                Start();
                ClearBest();
                FindSolutionPartial(10000, 1);

                Results.Add(mBestSolution);

                Opt.Add(nNumberOfNodes* nNumberOfSubraphs);
                Iter.Add(mBestIteration);
                iTime.Add(mBestCalcTime);
               




            }

        }


        public void Load(String FileName)
        {
            int oNumberOfSubgraphs;
            CalcType Temp;
            LoadFile(FileName, out oNumberOfSubgraphs);
            mPartitioning = new ABSBipartitioning(oNumberOfSubgraphs, mOriginalGraph);

            BFSNodeContainer.InitStatic(mOriginalGraph.NumberOfVertices);

            Temp = mCalcType;
            InitConst(mOriginalGraph, false);
            mCalcType = Temp;

            mPartitioning.SetRoots(mRoots);
            mInitialRoots = false;
        }

        public void InitTest(int iNumberOfSubgraphs, ABSGraph tGraph, bool[] iRoots) {

            int counter = 0;
            CalcType Temp;
            SetGraph(tGraph);
            mPartitioning = new ABSBipartitioning(iNumberOfSubgraphs, mOriginalGraph);
            mRoots = new int[iNumberOfSubgraphs];

            BFSNodeContainer.InitStatic(mOriginalGraph.NumberOfVertices);

            Temp = mCalcType;
            InitConst(mOriginalGraph, false);
            mCalcType = Temp;

            
            while(counter < iNumberOfSubgraphs)
            for (int i = 0; i < tGraph.NumberOfVertices; i++) {
                    if (iRoots[i]) {
                        mRoots[counter] = i;
                        counter++;
                    }

            }
            mPartitioning.SetRoots(mRoots);
            mInitialRoots = false;


        }
        public void LoadFile(String FileName, out int oNumberOfSubgraphs) {

            StreamReader File = new StreamReader(FileName);
            String temp;
            char[] delimiterChars = { ' ', ',', '.', ':', '\t' };
            string[] words;
            int tNumberOfSubgraphs;
            int tNumberOfNodes;

            temp = File.ReadLine();
            temp = File.ReadLine();

            tNumberOfNodes = Convert.ToInt16(temp);


            temp = File.ReadLine();
            temp = File.ReadLine();

            oNumberOfSubgraphs = Convert.ToInt16(temp);
            

            temp = File.ReadLine();
            temp = File.ReadLine();
            mMaxNodesInSubgraphs = Convert.ToInt16(temp);



            temp = File.ReadLine();
            mRoots = new int[oNumberOfSubgraphs];
            for (int i = 0; i < oNumberOfSubgraphs; i++) {
                temp = File.ReadLine();
                mRoots[i] = Convert.ToInt32(temp);

            }

            temp = File.ReadLine();
            mOriginalGraph = new ABSGraph();
            mOriginalGraph.Load(File);





           
            File.Close();

           
        }


        public int Seed {
            get { return mSeed; }
            set { mSeed = value; }
        }

        public int MaxNodesInSubgraph {
            get { return mMaxNodesInSubgraphs; }
            set { mMaxNodesInSubgraphs = value; }
        }
        public void SetAllMaxNodesInCycle() {

            mPartitioning.SetMaxNumberOFNodesInSubgraphs(mMaxNodesInSubgraphs);
            
        }

        public int NumberOfGeneratedSolutions
        {
            get { return mNumberOfGeneratedSolutions; }
            set {  mNumberOfGeneratedSolutions = value; }


        }


        public ABSBipartitioning BestPartitioning{
            get { return mBestPartitioning; }

        }

        public void SetGraph(ABSGraph iGraph)
        {
            mOriginalGraph = iGraph;

            mPartitioning.SetGraph(iGraph);
        }
        public int BestSolution {
            get { return mBestSolution; }
                
        }

        public int NumberOfSubgraphs
        {
            get { return mPartitioning.NumberOfSubgraphs; }
            set { mPartitioning.NumberOfSubgraphs = value; }

        }
        public ABSBipartitioning Partitioning
        {

            get { return mPartitioning; }
            set { mPartitioning = value; }
        }

        public void Start()
        {
            mPartitioning.mExceptCoef = mExceptCoef;
            mPartitioning.Start();
            mSizes = new List<int>();
            mTimes = new List<long>();
            


        }
        public void SetCurrentToBest() {
            mPartitioning = new ABSBipartitioning(mBestPartitioning);

        }
        public int TotalCovered() {

            int result=0;

            for (int i = 0; i < mPartitioning.NumberOfSubgraphs; i++) {
                result += mPartitioning.GetSubgraph(i).NumberOfCycleNodes();
            }

            return result;
        }

        public void RandomRoots() {
            mRoots = new int[mPartitioning.NumberOfSubgraphs];
            int counter = 0;
            int cT;
            bool newRoot;
            while (counter < mPartitioning.NumberOfSubgraphs) {

                cT = mGenerator.Next() % mPartitioning.GetGraph().NumberOfVertices;

                newRoot = true;
                for (int i = 0; i < counter; i++) {
                    if (cT == mRoots[i])
                        newRoot = false;
                }
                if (newRoot) {
                    mRoots[counter] = cT;
                    counter++;

                }

            }
            /*
            for (int i = 0; i < mPartitioning.NumberOfSubgraphs; i++) {

                            mRoots[i] = mGenerator.Next() % mPartitioning.NumberOfSubgraphs;
            }
            */
        }

        public void FindSolution(int NumberOfAttempts) {


            int cTotalCovered = 0;
            if (mInitialRoots)
                RandomRoots();
            mInitialRoots = false;
            mPartitioning.SetRoots(mRoots);
            for (int i = 0; i < NumberOfAttempts; i++)
            {
                NewGenerator(++mSeed);
                mPartitioning.SetGraph(mOriginalGraph);
                mPartitioning.CreateAllSubgraphs();
                mPartitioning.SetMaxNumberOFNodesInSubgraphs(mMaxNodesInSubgraphs);
                

                Start();

                MultiStep(1500);
                cTotalCovered = TotalCovered();
                if (mBestSolution < cTotalCovered) {

                    mBestSolution = cTotalCovered;
                    mPartitioning.CalculateSubgraphNeighbors(mOriginalGraph);
                    mBestPartitioning = new ABSBipartitioning(mPartitioning);
                    
                }
            }


        }
        public bool ImproveSolutionBasic() {
            int cTotalCovered;
            SetCurrentToBest();
            bool foundNew = true;
            bool Result = false;

            while (foundNew)
            {
                
                foundNew = false;
                while (mPartitioning.SimpleImprove(mOriginalGraph))
                {
                    foundNew = true;
                    Result = true;
                }
            }

            cTotalCovered = TotalCovered();
            if (mBestSolution < cTotalCovered)
            {

                mBestSolution = cTotalCovered;
                mBestPartitioning = new ABSBipartitioning(mPartitioning);
            }
            return Result;
        }

        public  List<int> GetRestartSubgraphsRandom(int NumberOfRestarts) {
            List<int> Result = new List<int>();
            List<int> SubComplete = new List<int>();
            List<int> SubOpen = new List<int>();
            List<int> SubAll = new List<int>();

            for (int i = 0; i < mPartitioning.NumberOfSubgraphs; i++) {

                if (!mPartitioning.GetSubgraph(i).IsComplete())
                    SubOpen.Add(i);
                SubAll.Add(i);
                
                
            }
            if (SubOpen.Count == 0)
                return Result;
            int oSelect = mGenerator.Next() % SubOpen.Count;
            int cSelect;

            Result.Add(SubOpen[oSelect]);
            SubAll.Remove(SubOpen[oSelect]);
            if (SubAll.Count == 0)
                return Result;
            while (Result.Count < NumberOfRestarts-1)
            {
                cSelect = mGenerator.Next() % (SubAll.Count);
                Result.Add(SubAll[cSelect]);
                SubAll.Remove(SubAll[cSelect]);
                if (SubAll.Count == 0)
                    break;
            }

            return Result;

            

            

            

        }

        public List<int> GetRestartSubgraphsNeighbor(int NumberOfRestarts)
        {
            List<int> Result = new List<int>();
            List<int> SubComplete = new List<int>();
            List<int> SubOpen = new List<int>();
            List<int> SubAll = new List<int>();
            List<int> Potential = new List<int>();

            List<int>[] SubNeigbors = mBestPartitioning.GetSubgraphNeighbors();

            if (SubNeigbors == null)
                return Result;
// collect Open
            for (int i = 0; i < mBestPartitioning.NumberOfSubgraphs; i++)
            {

                if (!mBestPartitioning.GetSubgraph(i).IsComplete())
                    SubOpen.Add(i);
                SubAll.Add(i);


            }
            // fisrt
            if (SubOpen.Count == 0)
                return Result;
            int oSelect = mGenerator.Next() % SubOpen.Count;
            int cSelect;
            int sSub;



            sSub = SubOpen[oSelect];

            Result.Add(sSub);

            
            SubOpen.Remove(sSub);


            foreach (int ns in SubNeigbors[sSub]) {

                if (!Potential.Contains(ns) && (ns != NumberOfSubgraphs))
                {
                    Potential.Add(ns);
                }
            }
            // loop
            

            while (Result.Count < NumberOfRestarts - 1) {

                if (Potential.Count == 0) 
                     break;

                
                oSelect = mGenerator.Next() % Potential.Count;
                sSub = Potential[oSelect];
                Potential.Remove(sSub);
                if (!Result.Contains(sSub)) {
                    Result.Add(sSub);
                }

                foreach (int ns in SubNeigbors[sSub])
                {

                    if ((!Potential.Contains(ns)) && (!Result.Contains(ns)) && (ns != NumberOfSubgraphs))
                    {
                        Potential.Add(ns);
                    }
                }


            }

            bool PotentialImprove = false;

            foreach (int t in Result) {
                if (SubNeigbors[NumberOfSubgraphs].Contains(t))
                {
                    PotentialImprove = true;
                    break;
                }
                    
            }

            if (!PotentialImprove)
                Result.Clear();

            return Result;







        }

        List<int> GetRestartSubgraphsNeighborFull(int tNumberOfRestarts)
        {
            List<int> RestartSub = new List<int>();
            int counter = 0;

            while ((tNumberOfRestarts <= NumberOfSubgraphs))
            {
                counter = 0;
                while ((RestartSub.Count == 0) && (counter < 350))
                {
                    RestartSub = GetRestartSubgraphsNeighbor(tNumberOfRestarts);
                    counter++;
                }
                if (counter < 350)
                    break;
                tNumberOfRestarts++;

            }

            return RestartSub;
        }

        List<int> GetRestartSubgraphs(int tNumberOfRestarts) {

            switch (mCalcType) {

                case CalcType.Random:
                    return GetRestartSubgraphsRandom(tNumberOfRestarts);
                case CalcType.Neighbor:


                    return GetRestartSubgraphsNeighborFull(tNumberOfRestarts);

            }
            return GetRestartSubgraphsRandom(tNumberOfRestarts);
        }

        public void FindSolutionPartial(int NumberOfAttempts, int NumberOfRestarts)
        {


            int cTotalCovered = 0;
            int tNumberOfRestarts;
            List<int> RestartSub = new List<int>();
            List<int> pRestartSub = new List<int>();
            bool[] oInCycle = new bool[mOriginalGraph.NumberOfVertices];
            bool[] cInCycle = new bool[mOriginalGraph.NumberOfVertices];
            int counter = 0;
            int ll;
            List<int>[] SubgraphNodes = null;
            bool First = true;


            mStopWatch = new Stopwatch();
            mStopWatch.Restart();

            mStopWatch.Start();

            if (mBestSolution == (mMaxNodesInSubgraphs) * NumberOfSubgraphs)
                return;


            if (mInitialRoots)
                RandomRoots();
            else {
                if(mBestPartitioning != null)
                mPartitioning = new ABSBipartitioning(mBestPartitioning, false);
                
            }
            mInitialRoots = false;

            mPartitioning.SetRoots(mRoots);
            


            for (int i = 0; i < NumberOfAttempts; i++)
            {
                //                if (i > 0)
                //                       mPartitioning = new ABSBipartitioning(mBestPartitioning, false);
                if (i - mBestIteration > 2000)
                    break;

                if (NumberOfGeneratedSolutions  > 0)
                {
                    foreach (int ss in RestartSub)
                    {

                        mPartitioning.CopySubgraph(mBestPartitioning, ss);
                    }

//                    mPartitioning.GetNodesInCycleStateAll(oInCycle);
                }
                /*              */
                if (NumberOfGeneratedSolutions > 0)
                    mPartitioning.FinishAll();


//                RegrowSize = Math.Min(NumberOfSubgraphs /3, 8);

         //       NewGenerator(++mSeed);



                tNumberOfRestarts = 2+mGenerator.Next()% RegrowSize;
//                tNumberOfRestarts = 2 + NumberOfSubgraphs/3;
                if (tNumberOfRestarts > NumberOfSubgraphs)
                    tNumberOfRestarts = NumberOfSubgraphs;



                pRestartSub = RestartSub;
                if (NumberOfGeneratedSolutions > 0)
                        RestartSub = GetRestartSubgraphs(tNumberOfRestarts);

                if(counter > 140)
                ll = 11;
                foreach (int cSelected in RestartSub)
                {

                    mPartitioning.RestartSubgraph(cSelected);
                    mPartitioning.StartParitial(cSelected, RestartSub);
                    
                    if (mPartitioning.GetSubgraph(cSelected).NumberOfCycleNodes() > 1)
                        ll = 0;


                }

                if (SubgraphNodes == null)
                {
                    mPartitioning.GetNodesInCycleStateAll(cInCycle);
                }
                else {

                    Array.Copy(oInCycle, cInCycle, mOriginalGraph.NumberOfVertices);

                    foreach (int ss in RestartSub) {

                        foreach (int v in SubgraphNodes[ss]) {

                            cInCycle[v] = false;
                        }
                        cInCycle[mRoots[ss]] = true;
                    }
                        
                }
                foreach (int ss in RestartSub)
                {

                        mPartitioning.GetSubgraph(ss).MakeUnavailable(cInCycle);
                }
                  
                if (NumberOfGeneratedSolutions == 0)
                {
                    mPartitioning.CreateAllSubgraphs();
                   
                    Start();
                }
                mPartitioning.SetAllGenerators();
                //               

                if (RestartSub.Count() > 0)
                    pRestartSub = RestartSub.ToList();
                else
                    pRestartSub = null;

//               if (mGenerator.Next() %2 ==1)
//                    MultiStep(158000, pRestartSub);
//                else
                      MultiStepSeq(158000, mGenerator.Next() %(ExpLength) +1, pRestartSub);
                

//                mPartitioning.CollectSubgraphNodes();


                cTotalCovered = TotalCovered();
                
                if (mBestSolution <=cTotalCovered)
                {
                    //                    if (mBestSolution > 147)
                    //                        return;
                    if (mBestSolution < cTotalCovered)
                    {
                        mBestIteration = i;
                        mBestCalcTime = mStopWatch.ElapsedMilliseconds;

                        mSizes.Add(cTotalCovered);
                        mTimes.Add(mBestCalcTime);
                    }

                    mBestSolution = cTotalCovered;
                    //if (mCalcType == CalcType.Neighbor){
                    

                        mPartitioning.CalculateSubgraphNeighbors(mOriginalGraph);
                        mPartitioning.AddAllSubgraphConnectionsForCluster();
//                    }

                    if (mBestPartitioning == null)
                        mBestPartitioning = new ABSBipartitioning(mPartitioning, false);
                    else {
//                        mBestPartitioning.Copy(mPartitioning);
//                        mBestPartitioning.Copy(mPartitioning, false);


                        foreach (int ss in RestartSub)
                        {

                            mBestPartitioning.CopySubgraph(mPartitioning, ss);
                        }
                    }

                    if (SubgraphNodes == null)
                    {
                        SubgraphNodes = new List<int>[NumberOfSubgraphs];

                        for (int ii = 0; ii < NumberOfSubgraphs; ii++)
                            SubgraphNodes[ii] = new List<int>();
                    }
//                    mBestPartitioning.CalculateSubgraphNeighbors(mOriginalGraph);
//                    mBestPartitioning.AddAllSubgraphConnectionsForCluster();
                    mBestPartitioning.GetNodesInCycleStateAll(oInCycle, SubgraphNodes);
                    //                    mPartitioning = new ABSBipartitioning(mBestPartitioning,false);
                    if (mBestSolution == (mMaxNodesInSubgraphs ) * NumberOfSubgraphs)
                        return;

                }
                NumberOfGeneratedSolutions++;
            }


        }
        public List<int> GetAllFree() {
            List<int> result = new List<int>();
            for (int i = 0; i < mOriginalGraph.NumberOfVertices; i++) {
                if (mPartitioning.GetSubgraphNode(i).SubgraphId == -1) {

                    result.Add(i);
                }
            }
            return result;
        }

        
        public void NewGenerator(int tSeed) {

            mSeed = tSeed;
            mGenerator = new Random(tSeed);
            mPartitioning.SetGenerator(mGenerator);
            mPartitioning.SetAllGenerators();

        }

        public BipartitioningProblem(int iNumberOfSubgraphs)
        {

            mPartitioning = new ABSBipartitioning(iNumberOfSubgraphs);
            mGenerator = new Random(mSeed);
            GenerateProblem();
            mPartitioning.SetGenerator(mGenerator);
            mPartitioning.CreateAllSubgraphs();
            mPartitioning.SetGenerator(mGenerator);
            mPartitioning.SetAllGenerators();
            mBestPartitioning = null;
            mCurrentSubgraph = 0;
            mInitialRoots =true;

            InitRegrowCoef();


        }

        public BipartitioningProblem()
        {


            mPartitioning = new ABSBipartitioning(8);
            mGenerator = new Random(mSeed);
            mPartitioning.SetGenerator(mGenerator);
            
            GenerateProblem();
            mRoots = new int[NumberOfSubgraphs];

            mPartitioning.AllocRoots();
            mPartitioning.CreateAllSubgraphs();
            mPartitioning.SetAllGenerators();
            mCurrentSubgraph = 0;
            mBestPartitioning = null;
            mInitialRoots = true;
            mNumberOfGeneratedSolutions = 0;

            InitRegrowCoef();
        }


        public BipartitioningProblem(ABSGraph tGraph) {

            mInitialRoots = false;
            mPartitioning = new ABSBipartitioning(50);

            InitRegrowCoef();
            /*
            mGenerator = new Random(mSeed);
            mPartitioning.SetGenerator(mGenerator);
            mPartitioning = new ABSB50ipartitioning(mPartitioning.NumberOfSubgraphs, tGraph);
            mPartitioning.SetGenerator(mGenerator);
            RandomRoots();
            mPartitioning.SetRoots(mRoots);
            mPartitioning.CreateAllSubgraphs();

            mCurrentSubgraph = 0;
            mBestPartitioning = null;
            mInitialRoots = true;
            mNumberOfGeneratedSolutions = 0;
            */
            InitConst(tGraph);
        }

        void InitRegrowCoef() {
            mExpLength  = 12;
            mRegrowSize = 8;
            
//            mExpLength = MaxNodesInSubgraph ;
            //            mRegrowSize = (int)Math.Min(Math.Ceiling(500 / (float)mMaxNodesInSubgraphs) +2, NumberOfSubgraphs);
//            mRegrowSize  = (int)Math.Min(Math.Ceiling(500 / (float)mMaxNodesInSubgraphs) + 2, NumberOfSubgraphs);
            
            mExceptCoef = (float)0.6;

        }

        void InitConst(ABSGraph tGraph, bool InitRandomRoots = true) {

            mGenerator = new Random(mSeed);
            mPartitioning.SetGenerator(mGenerator);
            mPartitioning = new ABSBipartitioning(mPartitioning.NumberOfSubgraphs, tGraph);
            mPartitioning.SetGenerator(mGenerator);
            if(InitRandomRoots)
                RandomRoots();
            mPartitioning.SetRoots(mRoots);
            mPartitioning.CreateAllSubgraphs();
            mCalcType = CalcType.Neighbor;
            mCurrentSubgraph = 0;
            mBestPartitioning = null;
            mInitialRoots = true;
            mNumberOfGeneratedSolutions = 0;

//            InitRegrowCoef();

        }

        public BipartitioningProblem(ABSGraph tGraph, int tNumberOfSabgraphs)
        {

            mInitialRoots = false;
            mPartitioning = new ABSBipartitioning(tNumberOfSabgraphs);
            InitConst(tGraph);

            InitRegrowCoef();
            /*
            mGenerator = new Random(mSeed);
            mPartitioning.SetGenerator(mGenerator);
            mPartitioning = new ABSBipartitioning(mPartitioning.NumberOfSubgraphs, tGraph);
            mPartitioning.SetGenerator(mGenerator);
            RandomRoots();
            mPartitioning.SetRoots(mRoots);
            mPartitioning.CreateAllSubgraphs();

            mCurrentSubgraph = 0;
            mBestPartitioning = null;
            mInitialRoots = true;
            mNumberOfGeneratedSolutions = 0;
            */
        }


        public void GenerateProblem()
        {

            ABSGraph tGraph;
            ABSGraphVisualiser tVisualiser;

            mGenerator = new Random(mSeed);
            double R = 0.072;

            ABSGraphGenerator.GeneratePlainDistanceMinDist(out tGraph, out tVisualiser, 38, 1, 1, R, R / 3, mGenerator);
            //            mGraph = new DFSGraph(tGraph);
            //            mVisualiser = new DFSGraphVisualiser(tVisualiser);

//            pVisualiser = new BipartitioningVisualiser(tVisualiser);


            Partitioning = new ABSBipartitioning(mPartitioning.NumberOfSubgraphs, tGraph);
            mPartitioning.SetGenerator(mGenerator);
            //   mVisualiser = new ProblemVisualiser(this, tVisualiser);


            //            mVisualiser.SerRGBForAll();

            //            mVisualiser.SetGraph(mGraph);
            //            mVisualiser.SetScreen(formGraphics2);


        }


        public void MultiStep(int NumberOfSteps, List<int> Active = null) {
            bool Finished;

            for (int i = 0; i < NumberOfSteps; i++) {
                Step(out Finished, Active);
                if (Finished)
                    return;
            }
                
        }
        public void MultiStepSeq(int NumberOfSteps, int period, List<int> Active = null)
        {
            bool change = false;
            int counter = 0;
            bool Finished;
            int ii = 0;
            for (int i = 0; i < NumberOfSteps; i++)
            {
                ii++;
                counter += StepSeq(change, out Finished, Active);
                if (Finished)
                    return;
                if (counter > period) {
                    counter = 0;
                    change = true;
                }
                else{

                    change = false;
                }

            }

        }


        public void Step( out bool Finished, List<int> Active = null)
        {

//            List<int> LastEar;
            BFSEar LastEar;
            int iSelect;
            int sSelect;
            Finished = false;
            if (!mPartitioning.GetSubgraph(mCurrentSubgraph).Finished)
            {
                //            if (mPartitioning.GetSubgraph(mCurrentSubgraph).StepEnumerateCycle() == CycleStepResult.CompletedCycle)
                if (mPartitioning.GetSubgraph(mCurrentSubgraph).StepEnumerateCycleRandom() == CycleStepResult.CompletedCycle)
                {

                    LastEar = mPartitioning.GetSubgraph(mCurrentSubgraph).GetLastEar();

                    foreach (int eN in LastEar.Nodes)
                    {

                        mPartitioning.DeleteTreeNodeInAllSubgraphs(eN, mCurrentSubgraph, Active);
                    }

                }
            }

            if (mPartitioning.GetSubgraph(mCurrentSubgraph).Finished) {
                if (Active != null)
                {
                    Active.Remove(mCurrentSubgraph);
                    if (Active.Count() == 0) {
                        Finished = true;
                    }
                }
            }
            if (Active == null)
                mCurrentSubgraph = (mCurrentSubgraph + 1) % NumberOfSubgraphs;
            else {

                
                    if (Active.Count() == 0)
                    {
                        Finished = true;
                    return;
                    }
                
                iSelect = mGenerator.Next() % Active.Count;
                mCurrentSubgraph = Active[iSelect];
            }

        }


        public int StepSeq(bool change, out bool Finished,  List<int> Active = null)
        {
            int result = 0;
            int iSelect;
//            List<int> LastEar;
            BFSEar LastEar;
            Finished = false;
            if (!mPartitioning.GetSubgraph(mCurrentSubgraph).Finished)
            {
                //            if (mPartitioning.GetSubgraph(mCurrentSubgraph).StepEnumerateCycle() == CycleStepResult.CompletedCycle)
                if (mPartitioning.GetSubgraph(mCurrentSubgraph).StepEnumerateCycleRandom() == CycleStepResult.CompletedCycle)
                {

                    LastEar = mPartitioning.GetSubgraph(mCurrentSubgraph).GetLastEar();

                    foreach (int eN in LastEar.Nodes)
                    {

                        mPartitioning.DeleteTreeNodeInAllSubgraphs(eN, mCurrentSubgraph, Active);
                    }
                    result = LastEar.Nodes.Count - 2;
                }
            }

            if ((mPartitioning.GetSubgraph(mCurrentSubgraph).Finished) || change)
            {
                if (Active == null)
                {

                    int nFinished = 0;
                    for (int i = 0; i < NumberOfSubgraphs; i++)
                    {
                        if (mPartitioning.GetSubgraph(i).Finished) {
                            nFinished++;
                        }
                        
                    }

                    if(nFinished == NumberOfSubgraphs)
                        Finished = true;
                    mCurrentSubgraph = (mCurrentSubgraph + 1) % NumberOfSubgraphs;
                }
                else {

                    if (mPartitioning.GetSubgraph(mCurrentSubgraph).Finished)
                    {
                        Active.Remove(mCurrentSubgraph);
                        if (Active.Count() == 0)
                        {
                            Finished = true;
                            return result;
                        }
                    }
                    iSelect = mGenerator.Next() % Active.Count;
                    mCurrentSubgraph = Active[iSelect];


                }
            }

            return result;
        }
        

    }
}
