﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System;
using System.IO;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace HamiltoninaGSD
{

    class BFSVisualiserInfo
    {

        public bool mShowBFS;
        public bool mShowBFSDistance;
        public bool mShowBFSParent;
        public bool mShowBFSTreeRoot;
        public bool mShowSelected;
        public bool mShowString;
        public bool mShowAllNodes;
        public bool mShowInCycleNode;


        public BFSVisualiserInfo()
        {

              mShowBFS = false;
              mShowBFSDistance = false;
              mShowBFSParent = false;
              mShowBFSTreeRoot = false;
              mShowSelected =false;
              mShowString = false;
              mShowAllNodes = false;
            mShowInCycleNode = true;
        }


    }
    class BipartitioningVisualiser
    {

        protected System.Drawing.Bitmap mBitmap;
        protected int mScreenX;
        protected int mScreenY;
        protected Graphics mScreen;
        protected int mCounter;
        protected int mFontSize;
        protected int mNodeSize;
        protected int mCut;

        protected int mBaseR;
        protected int mBaseG;
        protected int mBaseB;
        protected int mNumberOfVertices;

        protected int mSelectedIndex;

        public int GetSelected() { return mSelectedIndex; }


        public void SetRGB(int iR, int iG, int iB)
        {

            mBaseB = iB;
            mBaseG = iG;
            mBaseR = iR;
        }
        public void SetColorForIndex(int index) {
            if (index < 8)
            {
                mBaseB = (index & 1) * 255;
                mBaseG = ((index & 2) >> 1) * 255; ;
                mBaseR = ((index & 4) >> 2) * 255; ;
                return;
            }

            if (index < 16)
            {
                mBaseB = 30+(index & 1) * 155;
                mBaseG = 30 + ((index & 2) >> 1) * 155; 
                mBaseR = 30 + ((index & 4) >> 2) * 155; 
                return;
            }

            if (index < 24)
            {
                mBaseB = 60+(index & 1) * 105;
                mBaseG = 60 + ((index & 2) >> 1) * 105;
                mBaseR = 60 + ((index & 4) >> 2) * 105;
                return;
            }


            if (index < 30)
            {
                mBaseB = 90 + (index & 1) * 45;
                mBaseG = 90 + ((index & 2) >> 1) * 45;
                mBaseR = 90 + ((index & 4) >> 2) * 45;
                return;
            }


            if (index < 36)
            {
                mBaseB = 120 + (index & 1) * 125;
                mBaseG = 20 + ((index & 2) >> 1) * 125;
                mBaseR = 20 + ((index & 4) >> 2) * 125;
                return;
            }



        }

        //    protected bool mShowSelected;

        protected Kord[] mVertexPositions;

        public void Load(String FileName) {

            StreamReader File = new StreamReader(FileName);
            String      temp;
            char[] delimiterChars = { ' '};
             string[] words;

            temp = File.ReadLine();
            temp = File.ReadLine();

            mNumberOfVertices = Convert.ToInt16(temp);

            mVertexPositions = new Kord[mNumberOfVertices];

            temp = File.ReadLine();
            for (int i = 0; i < mNumberOfVertices; i++) {
                temp = File.ReadLine();
                words = temp.Split(delimiterChars);
                mVertexPositions[i] = new Kord(Convert.ToDouble(words[0]), Convert.ToDouble(words[1]));

            }

            File.Close();

            InitImageSpec();
        }


        public void InitImageSpec() {
            mBitmap = null;
            mScreenX = 1;
            mScreenY = 1;

            mFontSize = 10;
            mCounter = 0;
            mNodeSize = 10;
            mCut = 50;


            mBaseB = 0;
            mBaseG = 0;
            mBaseR = 255;
        }
        public BipartitioningVisualiser(String FileName) {

            Load(FileName);
        }
        public BipartitioningVisualiser(ABSGraphVisualiser A)
        {
            Init(A.mGraph);
            mNumberOfVertices = A.mGraph.NumberOfVertices;

            mVertexPositions = new Kord[A.mGraph.NumberOfVertices];
            mScreenX = A.mScreenX;
            mScreenY = A.mScreenY;
            for (int i = 0; i < A.mGraph.NumberOfVertices; i++)
            {

                mVertexPositions[i] = new Kord(A.mVertexPositions[i]);
            }
            // Init(A.mGraph);
        }

        public BipartitioningVisualiser(Kord[] nKord, int nNumberOfVertex) {

            mNumberOfVertices = nNumberOfVertex;
            mVertexPositions = new Kord[nNumberOfVertex];
            for (int i = 0; i < mNumberOfVertices; i++)
                mVertexPositions[i] = new Kord(nKord[i]);

            InitImageSpec();

        }
        public void Init(ABSGraph A)
        {
            mNumberOfVertices = A.NumberOfVertices;
            mVertexPositions = new Kord[A.NumberOfVertices];

            InitImageSpec();

        }


         public BipartitioningVisualiser()
        {
            InitImageSpec();

        }
        public void SetScreen(Graphics nScreen, int iCut = 50, System.Drawing.Bitmap iBitmap = null)
        {
            int mCut = iCut;
            double ScaleX = (nScreen.VisibleClipBounds.Width - mCut) / mScreenX;
            double ScaleY = (nScreen.VisibleClipBounds.Height - mCut) / mScreenY;


            mScreenX = Convert.ToInt16(nScreen.VisibleClipBounds.Width - mCut);
            mScreenY = Convert.ToInt16(nScreen.VisibleClipBounds.Height - mCut);

            ScaleAll(ScaleX, ScaleY);

            if (iBitmap == null)
                mBitmap = new Bitmap(mScreenX + mCut, mScreenY + mCut);
            else
                mBitmap = iBitmap;
            mScreen = nScreen;
            //            InitPositions();
            //            GeneratePositions();

        }
        public void DrawABSGraphNodes(ABSGraph mGraph,Color mColor, System.Drawing.Graphics tGraphics) {
            SolidBrush drawBrush = new SolidBrush(mColor);
            for (int i = 0; i < mGraph.NumberOfVertices; i++) {
//                tGraphics.FillRectangle(currentBrush, new Rectangle((int)mVertexPositions[index].mX - 5, (int)mVertexPositions[index].mY - 5, mNodeSize + 10, mNodeSize + 10));
                tGraphics.FillEllipse(drawBrush, new Rectangle((int)mVertexPositions[i].mX - 1, (int)mVertexPositions[i].mY - 1, mNodeSize + 2, mNodeSize + 2));


            }
        }

        public void DrawKordList(List<Kord> lList, System.Drawing.Graphics tGraphics) {
            int counter = 0;
            SolidBrush currentBrush;
            foreach (Kord a in lList) {
                currentBrush = new SolidBrush(Color.FromArgb(mBaseR, mBaseG, mBaseB));
                tGraphics.FillRectangle(currentBrush, new Rectangle((int)(a.mX * mScreenX) - 2, (int)(a.mY*mScreenY) - 2, mNodeSize + 4, mNodeSize +4));
            }
        }

        public void DrawKordListEdges(List<Kord> lList, double R, int width = 1)
        {
            int counter = 0;
            SolidBrush currentBrush = new SolidBrush(Color.FromArgb(mBaseR, mBaseG, mBaseB));
            
            Pen nodePen = new Pen(currentBrush, width);
            foreach (Kord a in lList)
                foreach (Kord b in lList)
                {
                    {
//                        currentBrush = new SolidBrush(Color.FromArgb(mBaseR, mBaseG, mBaseB));
                        

                        if (Kord.Distance(a,b) < R)
                        mScreen.DrawLine(nodePen, (int)((a.mX * mScreenX) - 2), (int)((a.mY * mScreenY) - 2),
                                                            (int)((b.mX * mScreenX) - 2), (int)((b.mY * mScreenY) - 2));

//                        tGraphics.FillRectangle(currentBrush, new Rectangle((int)(a.mX * mScreenX) - 2, (int)(a.mY * mScreenY) - 2, mNodeSize + 4, mNodeSize + 4));
                    }
                }
        }


        public void DrawKordListList(List<List<Kord>> lList)
        {
            int counter = 0;
            SolidBrush currentBrush;
            foreach (List<Kord> a in lList)
            {
                SetColorForIndex(counter++);
                DrawKordList(a, mScreen);

            }
        }

        public void DrawNodeBFS(int index, Color mColor, System.Drawing.Graphics tGraphics, BFSGraph mGraph, BFSVisualiserInfo dispBFSInfo)
        {


            Font drawFont = new Font("Arial", mFontSize);
            SolidBrush drawBrush = new SolidBrush(Color.Black);
            //            SolidBrush drawBrush = new SolidBrush(Color.FromArgb(mBaseR/2, mBaseG / 2, mBaseB / 2));
            SolidBrush currentBrush = new SolidBrush(Color.Pink);
            SolidBrush FinishedBrush = new SolidBrush(Color.Beige);
            SolidBrush fontBrush = new SolidBrush(Color.Black);
            string s;

            SizeF stringSize;
            SolidBrush nodeBrush = new SolidBrush(mColor);
            Pen nodePen = new Pen(nodeBrush, 3);


            if (((BFSGraph)mGraph).IsInCycle(index))
                drawBrush = new SolidBrush(Color.FromArgb(mBaseR, mBaseG, mBaseB));



            //            if (((BFSGraph)mGraph).IsFinished(index))
            //                tGraphics.FillRectangle(FinishedBrush, new Rectangle((int)mVertexPositions[index].mX - 6, (int)mVertexPositions[index].mY - 6, mNodeSize + 12, mNodeSize + 12));


            if (((BFSGraph)mGraph).IsLastUsed(index))
                tGraphics.FillRectangle(currentBrush, new Rectangle((int)mVertexPositions[index].mX - 5, (int)mVertexPositions[index].mY - 5, mNodeSize + 10, mNodeSize + 10));


            if (dispBFSInfo.mShowAllNodes)
                tGraphics.FillRectangle(drawBrush, new Rectangle((int)mVertexPositions[index].mX, (int)mVertexPositions[index].mY, mNodeSize, mNodeSize));
            if (((BFSGraph)mGraph).IsInCycle(index) == dispBFSInfo.mShowInCycleNode)
                tGraphics.FillRectangle(drawBrush, new Rectangle((int)mVertexPositions[index].mX, (int)mVertexPositions[index].mY, mNodeSize, mNodeSize));

            s = "";

            if (((BFSGraph)mGraph).Root == index)
            {
                fontBrush = new SolidBrush(Color.Red);
                drawFont = new Font("Arial", mFontSize + 2);
            }

            s = Convert.ToString(index);

            if (dispBFSInfo.mShowBFS)
                s += "/" + Convert.ToString(((BFSGraph)mGraph).GetBFS(index));
            if (dispBFSInfo.mShowBFSDistance)
                s += "/" + Convert.ToString(((BFSGraph)mGraph).GetBFSDistance(index));
            if (dispBFSInfo.mShowBFSParent)
                s += "/" + Convert.ToString(((BFSGraph)mGraph).GetBFSParent(index));
            if (dispBFSInfo.mShowBFSTreeRoot)
                s += "/" + Convert.ToString(((BFSGraph)mGraph).GetBFSTreeRoot(index));

            if (dispBFSInfo.mShowString)
                tGraphics.DrawString(s, drawFont, fontBrush, (float)mVertexPositions[index].mX, (float)mVertexPositions[index].mY);



        }

        public void DrawNodes(BFSGraph iGraph, System.Drawing.Graphics tGraphics, BFSVisualiserInfo dispBFSInfo)
        {

            for (int i = 0; i < iGraph.NumberOfVertices; i++)
            {
                DrawNodeBFS(i, Color.Black, tGraphics, iGraph, dispBFSInfo);
            }

        }


        public void DrawEdge(int i, int j, Color ccolor, System.Drawing.Graphics tGraphics, int iWidth = 3)
        {



            SolidBrush nodeBrush = new SolidBrush(ccolor);
            Pen nodePen = new Pen(nodeBrush, iWidth);

            tGraphics.DrawLine(nodePen, (int)(mVertexPositions[i].mX + mNodeSize / 2), (int)(mVertexPositions[i].mY + mNodeSize / 2),
                                                (int)(mVertexPositions[j].mX + mNodeSize / 2), (int)(mVertexPositions[j].mY + mNodeSize / 2));


        }
        public void DrawEdges(ABSGraph iGraph, System.Drawing.Graphics tGraphics)
        {


            Font drawFont = new Font("Arial", mFontSize);
            SolidBrush drawBrush = new SolidBrush(Color.Black);
            //            SolidBrush currentBrush = new SolidBrush(Color.FromArgb(mBaseR, mBaseG, mBaseB));
            SolidBrush currentBrush = new SolidBrush(Color.FromArgb(mBaseR, mBaseG, mBaseB));


            SolidBrush nodeBrush = new SolidBrush(Color.Black);
            Pen nodePen = new Pen(nodeBrush, 3);


            string s;
            //            mNodeSize = 50;


            SizeF stringSize;

            for (int i = 0; i < iGraph.NumberOfVertices; i++)
                for (int j = i + 1; j < iGraph.NumberOfVertices; j++)
                    if (iGraph.EdgeExists(i, j))
                    {

                        DrawEdge(i, j, Color.LightGray, tGraphics);
                        /*    tGraphics.DrawLine(nodePen, mVertexPositions[i].mX + mNodeSize / 2, mVertexPositions[i].mY + mNodeSize / 2,
                                                        mVertexPositions[j].mX + mNodeSize / 2, mVertexPositions[j].mY + mNodeSize / 2);
                         * */
                    }

            nodePen.Brush = currentBrush;

            drawBrush.Dispose();
            drawFont.Dispose();
            nodePen.Dispose();
            nodeBrush.Dispose();



        }

        public void DrawTreeBFS(int index, Color iColor, BFSGraph iGraph, System.Drawing.Graphics tGraphics)
        {

            List<int> Descendats;
            Descendats = ((BFSGraph)iGraph).GetDescendants(index);
            if (Descendats != null)
            {
                foreach (int v in Descendats)
                {

                    DrawEdge(index, v, iColor, tGraphics, 6);
                    DrawTreeBFS(v, iColor, iGraph, tGraphics);
                }
            }

        }


        public void DrawEarsBFS(Color iColor, BFSGraph iGraph, System.Drawing.Graphics tGraphics)
        {

            List<BFSEar> tEars;
            int a, b;

            if (((BFSGraph)iGraph).GetEars() == null)
                return ;
            tEars = ((BFSGraph)iGraph).GetEars().Ears;

            foreach (BFSEar tEar in tEars)
            {
                for (int i = 0; i < tEar.Nodes.Count - 1; i++)
                {
                    if (tEar.Descendants != null)
                    {
                        if (tEar.Descendants.Count > 0)
                            DrawEdge(tEar.Nodes[i], tEar.Nodes[i + 1], iColor, tGraphics, 1);
                        else
                            DrawEdge(tEar.Nodes[i], tEar.Nodes[i + 1], iColor, tGraphics, 5);
                    }
                }


            }

        }

        public void DrawBFSGraph(System.Drawing.Graphics tGraphics, BFSGraph iGraph, bool clear = true)
        {

            BFSVisualiserInfo dispInfo = new BFSVisualiserInfo();

            if (clear)
                tGraphics.Clear(Color.White);
            DrawEdges(iGraph, tGraphics);
            //            DrawPartitioningEdges(tGraphics);
            //        DrawSelected(tGraphics);
            DrawTreeBFS(((BFSGraph)iGraph).Root, Color.FromArgb((int)(mBaseR * 0.25) + 180, (int)(int)(mBaseG * 0.25) + 180, (int)(int)(mBaseB * 0.25) + 180), iGraph, tGraphics);
            DrawEarsBFS(Color.FromArgb((int)(mBaseR * 0.1) + 200, (int)(mBaseG * 0.5) + 100, (int)(mBaseB * 0.1) + 200), iGraph, tGraphics);


            DrawNodes(iGraph, tGraphics, dispInfo);
            //            DrawPartitioning(tGraphics);
            //      mScreen.DrawImage(mBitmap, 0, 0);
            //            tGraphics.Dispose();

        }
        public void SetVertexCord(Kord iPos, int index) { mVertexPositions[index] = new Kord(iPos); }
        public void SetVertexCord(double iX, double iY, int index) { mVertexPositions[index] = new Kord(iX, iY); }
        public void ScaleAll(double ScaleX, double ScaleY)
        {


            for (int i = 0; i < mNumberOfVertices; i++)
            {
                mVertexPositions[i].mX *= ScaleX;
                mVertexPositions[i].mY *= ScaleY;

            }
        }


        public void DrawParititioning(ABSBipartitioning iPart)
        {

            System.Drawing.Graphics tGraphics = Graphics.FromImage(mBitmap);
            DrawParititioning(iPart, tGraphics);
            mScreen.DrawImage(mBitmap, 0, 0);
        }

        public void DrawParititioning(ABSBipartitioning iPart, System.Drawing.Graphics tGraphics) {

            BFSGraph cGraph;
            BFSVisualiserInfo dispInfo = new BFSVisualiserInfo();
            BFSVisualiserInfo dispInfoA = new BFSVisualiserInfo();

            tGraphics.Clear(Color.White);

            dispInfoA.mShowAllNodes = true;
            

            DrawEdges(iPart.GetGraph(), tGraphics);
           // DrawEdges(iPart.GetSubgraph(0), tGraphics);
            //            DrawPartitioningEdges(tGraphics);
            //        DrawSelected(tGraphics);


            
            for (int i = 0; i < iPart.NumberOfSubgraphs; i++) {
                cGraph = iPart.GetSubgraph(i);
                SetColorForIndex(i);
                DrawTreeBFS(((BFSGraph)cGraph).Root, Color.FromArgb((int)(mBaseR * 0.25) + 180, (int)(int)(mBaseG * 0.25) + 180, (int)(int)(mBaseB * 0.25) + 180), cGraph, tGraphics);
            }
            DrawABSGraphNodes(iPart.GetGraph(), Color.Gold, tGraphics);


            for (int i = 0; i < iPart.NumberOfSubgraphs; i++)
            {
                cGraph = iPart.GetSubgraph(i);
                SetColorForIndex(i);
//                DrawEarsBFS(Color.FromArgb((int)(mBaseR * 0.1) + 200, (int)(mBaseG * 0.5) + 100, (int)(mBaseB * 0.1) + 200), cGraph, tGraphics);
                DrawEarsBFS(Color.FromArgb(0, 0, 0), cGraph, tGraphics);
                DrawNodes(cGraph, tGraphics, dispInfo);
            }
         
            for (int i = 0; i < iPart.NumberOfSubgraphs; i++)
            {
                cGraph = iPart.GetSubgraph(i); 
                SetColorForIndex(i);
                /*
                if (i == 0)
                {
                    dispInfo.mShowBFSDistance = true;
                    dispInfo.mShowString = true;
                }
                else
                {
                    dispInfo.mShowBFSDistance = false;
                    dispInfo.mShowString = false;
                }
                */
                DrawNodes(cGraph, tGraphics, dispInfo);
            }
            
            for (int i = 0; i < iPart.NumberOfSubgraphs; i++)
            {
                cGraph = iPart.GetSubgraph(i);
                SetColorForIndex(i);
                DrawRootNode(cGraph, i,tGraphics);
            }


        }

        public void DrawRootNode(BFSGraph cGraph, int index,System.Drawing.Graphics tGraphics){

            Font drawFont = new Font("Arial", mFontSize);
            SolidBrush fontBrush = new SolidBrush(Color.Black);
            SolidBrush currentBrush = new SolidBrush(Color.FromArgb(125 + mBaseR/2, 125 + mBaseG/2, 125 + mBaseB/2));
            string s;




            


            s = "" + cGraph.NumberOfCycleNodes()+ "/" +index;

            
                tGraphics.FillRectangle(currentBrush, new Rectangle((int)mVertexPositions[cGraph.Root].mX - 8, (int)mVertexPositions[cGraph.Root].mY - 8, mNodeSize + 16, mNodeSize + 16));


            tGraphics.DrawString(s, drawFont, fontBrush, (float)mVertexPositions[cGraph.Root].mX - 5, (float)mVertexPositions[cGraph.Root].mY - 5);


        }
        /*

        public override void DrawTest(bool clear = true)
        {

            System.Drawing.Graphics tGraphics = Graphics.FromImage(mBitmap);
            DrawTest(tGraphics, clear);

            /*
                         tGraphics.Clear(Color.White);
                         DrawContainerBay(tGraphics);
                         mScreen.DrawImage(mBitmap, 0, 0);
                         tGraphics.Dispose();
             * 
            */
        /*
                    if (clear)
                        tGraphics.Clear(Color.White);
                    DrawEdges(tGraphics);
                    //            DrawPartitioningEdges(tGraphics);
                    DrawSelected(tGraphics);
                    DrawTree(((BFSGraph)mGraph).Root, Color.FromArgb((int)(mBaseR * 0.25)+ 180, (int)(int)(mBaseG * 0.25) + 180, (int)(int)(mBaseB * 0.25) + 180),  tGraphics);
                    DrawEars(Color.FromArgb((int)(mBaseR * 0.1) + 200, (int)(mBaseG * 0.5)+100, (int)(mBaseB * 0.1) + 200), tGraphics);


                        DrawNodes(tGraphics);
                    //            DrawPartitioning(tGraphics);
                    mScreen.DrawImage(mBitmap, 0, 0);
        //            tGraphics.Dispose();


    }

    */

        public void Select(int sX, int sY, ABSGraph iGraph)
        {
            double minDist = double.MaxValue;
            double cDist = 0;

            //            Kord A = new Kord(sX - mCut, sY - mCut );
            Kord A = new Kord(sX, sY);
            mSelectedIndex = -1;
            for (int i = 0; i < iGraph.NumberOfVertices; i++)
            {
                cDist = Kord.Distance(A, mVertexPositions[i]);

                if (cDist < minDist)
                {
                    mSelectedIndex = i;
                    minDist = cDist;

                }

            }


        }

     //   public int GetSelected() { return mSelectedIndex; }
    }
}
