﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace HamiltoninaGSD
{
    public partial class Form1 : Form
    {
        ABSGraph OriginalGraph;
        BipartitioningVisualiser mVisualiser;
        Random mGenerator;
        BipartitioningProblem mProblem;
        int mSeed;
        string[] mPowerMatFileNames;
        int mNumberOfPowerMatFileNames;

        const int NumNodes = 5;
        const int NumGraphs = 5;

        public Form1()
        {
            InitializeComponent();
            Initialize();
        }


        public void Initialize()
        {

            //            mProblem = new GraphPartitioningProblemSD();
            //            mProblem = new BalancedGraphPartitioningProblemSD();
            OriginalGraph = new DFSGraph(10);
            ABSGraph tGraph;
            ABSGraphVisualiser tVisualiser;
            int NumberOfNodes = 150;

            BFSNodeContainer.InitStatic(NumberOfNodes);
            mSeed = 14;

            //            mVisualiser = new DFSGraphVisualiser(mGraph);


            //            mVisualiser = new BFSGraphVisualiser(mGraph);


            System.Drawing.Graphics formGraphics2 = pictureBox1.CreateGraphics();
            mGenerator = new Random(mSeed);
            double R = 0.045;

            ABSGraphGenerator.GeneratePlainDistanceMinDist(out tGraph, out tVisualiser, NumberOfNodes, 1, 1, R, R / 3, mGenerator);
            //            mGraph = new DFSGraph(tGraph);
            //            mVisualiser = new DFSGraphVisualiser(tVisualiser);
            mVisualiser = new BipartitioningVisualiser(tVisualiser);
            mProblem = new BipartitioningProblem();


            mProblem.Load("test.pro");
            mVisualiser = new BipartitioningVisualiser("test.vis");

            mVisualiser.SetScreen(formGraphics2);



            OriginalGraph = new BFSGraph(tGraph);

            propertyGrid1.SelectedObject = mProblem;
        }


        private void pictureBox1_Click(object sender, EventArgs e)
        {

        }

        private void pictureBox1_DoubleClick(object sender, EventArgs e)
        {

            int X, Y;

            System.Drawing.Graphics formGraphics2 = pictureBox1.CreateGraphics();
            mVisualiser.SetScreen(formGraphics2);



            X = ((MouseEventArgs)e).X;
            Y = ((MouseEventArgs)e).Y;


            mVisualiser.Select(X, Y, mProblem.BestPartitioning.GetGraph());
            int SelectedNode = mVisualiser.GetSelected();

            //            mProblem.Partitioning.CollectSubgraphNodes();
            //            mProblem.SetCurrentToBest();
            mProblem.Partitioning.CollectSubgraphNodes();
            mProblem.Partitioning.RemoveNodeFromCycles(SelectedNode);


            //            System.Drawing.Graphics formGraphics2 = pictureBox1.CreateGraphics();
            //            mProblem.MultiStep(1000);



            //            mVisualiser.SetScreen(formGraphics2);
            mVisualiser.DrawParititioning(mProblem.Partitioning);

            //((BFSGraph)mGraph).DeleteTreeNode(mVisualiser.GetSelected(), true);

            //((BFSGraphVisualiser)mVisualiser).DrawTest();

        }



        public void GetStats(List<int> Solutions, List<int> Opt, out double AvgError, out double MaxError, out double Stdev)
        {


            List<double> ErrorPrecent = new List<double>();

            int NumberOfProblems = Solutions.Count;
            for (int i = 0; i < Solutions.Count; i++)
            {

                ErrorPrecent.Add(100 * (Opt[i] - Solutions[i]) / (double)Opt[i]);
            }

            Stdev = 0;

            AvgError = ErrorPrecent.Average();
            MaxError = ErrorPrecent.Max();

            for (int i = 0; i < Solutions.Count; i++)
            {

                Stdev += (ErrorPrecent[i] - AvgError) * (ErrorPrecent[i] - AvgError);
            }
            Stdev = Math.Sqrt(Stdev / NumberOfProblems);


        }

        public void GetStatsSimp(List<int> Solutions, out double Avg, out double Max, out double Stdev)
        {


            List<double> SolVal = new List<double>();

            int NumberOfProblems = Solutions.Count;
            for (int i = 0; i < Solutions.Count; i++)
            {

                SolVal.Add(Solutions[i]);
            }

            Stdev = 0;

            Avg = SolVal.Average();
            Max = SolVal.Max();

            for (int i = 0; i < Solutions.Count; i++)
            {

                Stdev += (Solutions[i] - Avg) * (Solutions[i] - Avg);
            }
            Stdev = Math.Sqrt(Stdev / NumberOfProblems);


        }



        private void button1_Click(object sender, EventArgs e)
        {
            mProblem.ImproveSolutionBasic();

            if (mProblem.BestPartitioning != null)
            {


                //                mVisualiser.DrawParititioning(mProblem.BestPartitioning);
            }

            propertyGrid1.Refresh();

        }

        private void buttonBestToCurrent_Click(object sender, EventArgs e)
        {
            mProblem.Partitioning.CollectSubgraphNodes();
            mProblem.SetCurrentToBest();
            mProblem.Partitioning.CollectSubgraphNodes();


        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            mProblem.ImproveSolutionBasic();

            if (mProblem.BestPartitioning != null)
            {


                mVisualiser.DrawParititioning(mProblem.BestPartitioning);
            }

            propertyGrid1.Refresh();
        }

        private void buttonSolve_Click_1(object sender, EventArgs e)
        {

            mProblem.NumberOfGeneratedSolutions = 0;
            mProblem.SetAllMaxNodesInCycle();
            mProblem.Start();



            for (int i = 0; i < 1; i++)
            {
                mProblem.FindSolutionPartial(25000, 1);
                //                Problem.FindSolution(100);

                if (mProblem.BestPartitioning != null)
                {


                    //                    mVisualiser.DrawParititioning(mProblem.BestPartitioning);
                }

                propertyGrid1.Refresh();
            }
            if (mProblem.BestPartitioning != null)
                mVisualiser.DrawParititioning(mProblem.BestPartitioning);

            mProblem.SaveIntermediateSolutions("Inter");
        }

        private void Step_Click_1(object sender, EventArgs e)
        {
            System.Drawing.Graphics formGraphics2 = pictureBox1.CreateGraphics();

            mProblem.SetAllMaxNodesInCycle();
            mProblem.Start();

            mProblem.MultiStep(10);



            mVisualiser.SetScreen(formGraphics2);
            mVisualiser.DrawParititioning(mProblem.Partitioning);
        }

        private void buttonTest_Click_1(object sender, EventArgs e)
        {
            System.Drawing.Graphics formGraphics2 = pictureBox1.CreateGraphics();
            //        int tN = mProblem.MaxNodesInSubgraph;
            //            mProblem = new BipartitioningProblem(OriginalGraph, mProblem.NumberOfSubgraphs);
            //            mProblem.SetGraph(OriginalGraph);
            propertyGrid1.SelectedObject = mProblem;
            mProblem.NewGenerator(mSeed++);

            //            mProblem.MaxNodesInSubgraph = tN;
            mProblem.SetAllMaxNodesInCycle();
            mProblem.Start();

            mVisualiser.SetScreen(formGraphics2);
            mVisualiser.DrawParititioning(mProblem.Partitioning);
        }


        void GenerateAndSaveProblemInstance(int tNumberOfSubgraphs, int tNumberOfNodes, float alpha, int tSeed, String RootDir) {

            ProblemGenerator tProblemGenerator = new ProblemGenerator();
            ProblemInstance tProblemInstance;

            double RConnect = 1 / Math.Sqrt(alpha * tNumberOfNodes * tNumberOfSubgraphs);
            double RFree = RConnect / 30;
            double tBoxXS = 1;
            double tBoxYS = 1;
            List<List<Kord>> tProblemCords;
            ProblemInstance Instance;

            tProblemGenerator.GenerateProblemInstance(tNumberOfSubgraphs, tNumberOfNodes, RConnect, RFree, tBoxXS, tBoxYS, tSeed, out Instance);


            String FileName = RootDir + "_" + tNumberOfSubgraphs + "_" + tNumberOfNodes + "_" + tSeed;
            Instance.SaveComplete(FileName);

        }

        private void GeneraterProblem_Click(object sender, EventArgs e)
        {
            ProblemGenerator tProblemGenerator = new ProblemGenerator();
            System.Drawing.Graphics formGraphics2 = pictureBox1.CreateGraphics();
            ProblemInstance tProblemInstance;

            int tNumberOfNodes = 10;
            int tNumberOfSubgraphs = 50;

            int tInitialNumberOfNodes = 35;
            double RConnect = 1 / Math.Sqrt(2 * tNumberOfNodes * tNumberOfSubgraphs);
            double RFree = RConnect / 30;
            double tBoxXS = 1;
            double tBoxYS = 1;
            mSeed++;
            List<List<Kord>> tProblemCords;
            ProblemInstance Instance;


            //            tProblemGenerator.GenerateProblem(tNumberOfNodes, tInitialNumberOfNodes, RConnect, RFree, tBoxXS, tBoxYS, tSeed);

            //            tProblemCords = tProblemGenerator.GenerateProblemInstanceList(tNumberOfSubgraphs, tNumberOfNodes, RConnect, RFree, tBoxXS, tBoxYS, mSeed);

            tProblemGenerator.GenerateProblemInstance(tNumberOfSubgraphs, tNumberOfNodes, RConnect, RFree, tBoxXS, tBoxYS, mSeed, out Instance);

            Instance.SaveComplete("test");

            List<Kord> kList = Instance.Kords.ToList();

            mVisualiser = tProblemGenerator.mVisualiser;
            mVisualiser = new BipartitioningVisualiser(Instance.Kords, tNumberOfSubgraphs * tNumberOfNodes);
            mVisualiser = new BipartitioningVisualiser("test.vis");

            mProblem = new BipartitioningProblem();

            mProblem.Load("test.pro");
            mVisualiser.SetScreen(formGraphics2);

            formGraphics2.Clear(Color.White);
            /*   List<Kord> All = new List<Kord>();
               foreach (List<Kord> l in tProblemCords)
               {
                //   mVisualiser.DrawKordListEdges(l, 2 * RConnect);
                   All.AddRange(l.ToArray());
               }
               */
            //            mVisualiser.SetRGB(125, 125, 125);
            //            mVisualiser.DrawKordListEdges(All, 2 * RConnect);

            mVisualiser.SetRGB(125, 125, 125);
            mVisualiser.DrawKordListEdges(kList, 2 * RConnect);

            for (int i = 0; i < tNumberOfSubgraphs; i++)
            {
                mVisualiser.SetColorForIndex(i);
                mVisualiser.DrawKordListEdges(Instance.GetPartitionKords(i), 2 * RConnect, 3);
            }
            /*            mVisualiser.SetRGB(255, 0, 0);
                        foreach (List<Kord> l in tProblemCords)
                        {
                            mVisualiser.DrawKordListEdges(l, 2 * RConnect);
            //                All.AddRange(l.ToArray());
                        }
                        mVisualiser.DrawKordListList(tProblemCords);
                        */
            //            tProblemInstance = new ProblemInstance()
            //            mVisualiser.DrawParititioning(tProblemGenerator.mPartitioning);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            int[] NumberOfNodes = { 5, 10, 25, 50, 100 };
            int[] NumberOfSubgraphs = { 5, 10, 25, 50, 100 };
            int NumberOfInstances = 40;
            String RootDir = ".\\Basic\\Alpha2\\prob";

            for (int i = 0; i < NumberOfNodes.Length; i++)
                for (int j = 0; j < NumberOfSubgraphs.Length; j++)
                    for (int k = 0; k < NumberOfInstances; k++)
                        GenerateAndSaveProblemInstance(NumberOfSubgraphs[j], NumberOfNodes[i], 2, k, RootDir);

            RootDir = ".\\Basic\\Alpha15\\prob";

            for (int i = 0; i < NumberOfNodes.Length; i++)
                for (int j = 0; j < NumberOfSubgraphs.Length; j++)
                    for (int k = 0; k < NumberOfInstances; k++)
                        GenerateAndSaveProblemInstance(NumberOfSubgraphs[j], NumberOfNodes[i], (float)1.5, k, RootDir);

        }

        public void DataSets(out int[] aNumberOfNodes, out int[] aNumberOfSubgraphs)
        {


            int index = 0;


            aNumberOfNodes = new int[NumNodes];
            aNumberOfSubgraphs = new int[NumGraphs];


            index = 0;
            aNumberOfNodes[index++] = 5;
            aNumberOfNodes[index++] = 10;
            aNumberOfNodes[index++] = 25;
            aNumberOfNodes[index++] = 50;
            aNumberOfNodes[index++] = 100;
            //    aNumSupply[index++] = 200;
            //   aNumSupply[index++] = 400;

            index = 0;
            aNumberOfSubgraphs[index++] = 5;
            aNumberOfSubgraphs[index++] = 10;
            aNumberOfSubgraphs[index++] = 25;
            aNumberOfSubgraphs[index++] = 50;
            aNumberOfSubgraphs[index++] = 100;

        }

        private void buttonSolveSet_Click(object sender, EventArgs e)
        {
            List<int> Solutions;
            List<int> Opt;
            List<int> Iter;
            List<long> Time;

            List<double> ErrorPrecent = new List<double>();
            double AvgError;
            double MaxError;
          
            double Stdev;
            BipartitioningProblem lProblem;
            string Folder = ".\\Basic\\Alpha2\\";
            //            GraphPartitioningProblemSD tProblem = new GraphPartitioningProblemSD();
            BFSNodeContainer.InitStatic(150);
            lProblem = new BipartitioningProblem();
            lProblem.CalcType = CalcType.Neighbor;

            //            mProblem.SolveProblemSet(10,10, )

            lProblem.SolveProblemSet(10, 10, out Solutions, out Opt, out Iter, out Time, Folder);
            GetStats(Solutions, Opt, out AvgError, out MaxError, out Stdev);

        }


     


        public void CalcStandard(string Folder, BipartitioningProblem Problem, int StartSubgraph, int EndSubgraph, int StartNode, int EndNode)
        {

         

            int nNumberOfNodes;
            int nNumberOfSubgraphs;
            int[] aNumberOfNodes;
            int[] aNumberOfSubgraphs;

            //      int nRandomSeed = 1;


            DataSets(out aNumberOfNodes, out aNumberOfSubgraphs);

            List<int> Solutions = new List<int>();
            List<int> Opt = new List<int>();
            List<int> Iter = new List<int>();
            List<long> CalcTime = new List<long>();


            //            ABSBipartitioning.ExceptCoef = (float)0.5;

          

            System.IO.StreamWriter fileSolutions;
            BFSNodeContainer.InitStatic(10000);

            for (int i = StartSubgraph; i < EndSubgraph; i++)
            {
                for (int j = StartNode; j < EndNode; j++)
                {

                    nNumberOfNodes = aNumberOfNodes[j];
                    nNumberOfSubgraphs = aNumberOfSubgraphs[i];
                    Problem.SolveProblemSet(nNumberOfNodes, nNumberOfSubgraphs, out Solutions, out Opt, out Iter, out CalcTime, Folder);

                    fileSolutions = new System.IO.StreamWriter(Folder+"Sol\\" + "Pro" + nNumberOfSubgraphs + "X" + nNumberOfNodes + Problem.GetNameCalc() + ".txt");
                    for (int ii = 0; ii < Solutions.Count; ii++) {

                        fileSolutions.WriteLine(Solutions[ii] + " " + Iter[ii] + " " + CalcTime[ii] + " "  + (nNumberOfNodes* nNumberOfSubgraphs));

                    }

                    fileSolutions.Close();

                }
            }

        }



        public void StatsStandard(string Folder)
        {



            int nNumberOfNodes;
            int nNumberOfSubgraphs;
            int[] aNumberOfNodes;
            int[] aNumberOfSubgraphs;


            BipartitioningProblem[] Problems = new BipartitioningProblem[2];

            Problems[0] = new BipartitioningProblem();
            Problems[1] = new BipartitioningProblem();
//            Problems[2] = new BipartitioningProblem();

            Problems[0].CalcType = CalcType.Neighbor;
            Problems[1].CalcType = CalcType.Random;

//            Problems[0].ExceptCoef = (float)0.6;
//            Problems[1].ExceptCoef = (float)0.8;
            //            Problems[2].CalcType = CalcType.Neighbor;

            string s;
            string stProblem;


            System.IO.StreamWriter fileSolutions;

            //      int nRandomSeed = 1;


            DataSets(out aNumberOfNodes, out aNumberOfSubgraphs);

            List<int> Solutions = new List<int>();
            List<int> Opt = new List<int>();
            List<int> Iter = new List<int>();
            List<long> CalcTime = new List<long>();


            //            ABSBipartitioning.ExceptCoef = (float)0.5;



            BFSNodeContainer.InitStatic(10000);


            fileSolutions = new System.IO.StreamWriter("testc.txt");

            
            fileSolutions.Close();


            for (int i = 0; i < NumGraphs; i++)
            {
                for (int j = 0; j < NumNodes; j++)
                {


                     Problems = new BipartitioningProblem[2];

                    Problems[0] = new BipartitioningProblem();
                    Problems[1] = new BipartitioningProblem();
//                    Problems[2] = new BipartitioningProblem();

                    Problems[0].CalcType = CalcType.Random;
                    Problems[1].CalcType = CalcType.Neighbor;
//                    Problems[2].CalcType = CalcType.Neighbor;

                    /*
                                        Problems[0].ExceptCoef = (float)0.8;
                                        Problems[1].ExceptCoef = (float)0.6;
                                        Problems[2].ExceptCoef = (float)0.4;
                    /*                    */

                   
                    /*Problems[0].ExpLength =16;
                    Problems[1].ExpLength = 12;
                    Problems[2].ExpLength = 8;
                    */
                    
/*
                   Problems[0].RegrowSize = 4;
                   Problems[1].RegrowSize =8;
                   Problems[2].RegrowSize = 12;
                    /*                   */
                    s = "a";


                    nNumberOfNodes = aNumberOfNodes[j];
                    nNumberOfSubgraphs = aNumberOfSubgraphs[i];

                    stProblem = Folder +"\\Sol\\Pro" + nNumberOfSubgraphs + "X" + nNumberOfNodes;




                    s = PrintStats(stProblem, Problems, 2);
                    //                    stProblem = 



                    fileSolutions = new System.IO.StreamWriter("testc.txt", true);

                    fileSolutions.WriteLine(nNumberOfSubgraphs + "X" + nNumberOfNodes + " & " +s);
                    fileSolutions.Close();


                   

                }
            }

        }

        public void StatsStandardCompare(string Folder)
        {



            int nNumberOfNodes;
            int nNumberOfSubgraphs;
            int[] aNumberOfNodes;
            int[] aNumberOfSubgraphs;


            BipartitioningProblem[] Problems = new BipartitioningProblem[2];

            Problems[0] = new BipartitioningProblem();
            Problems[1] = new BipartitioningProblem();
            //            Problems[2] = new BipartitioningProblem();

            Problems[0].CalcType = CalcType.Neighbor;
            Problems[1].CalcType = CalcType.Random;

            //            Problems[0].ExceptCoef = (float)0.6;
            //            Problems[1].ExceptCoef = (float)0.8;
            //            Problems[2].CalcType = CalcType.Neighbor;

            string s;
            string stProblem;


            System.IO.StreamWriter fileSolutions;

            //      int nRandomSeed = 1;


            DataSets(out aNumberOfNodes, out aNumberOfSubgraphs);

            List<int> Solutions = new List<int>();
            List<int> Opt = new List<int>();
            List<int> Iter = new List<int>();
            List<long> CalcTime = new List<long>();


            //            ABSBipartitioning.ExceptCoef = (float)0.5;



            BFSNodeContainer.InitStatic(10000);


            fileSolutions = new System.IO.StreamWriter("testCompare.txt");


            fileSolutions.Close();


            for (int i = 0; i < NumGraphs-1; i++)
            {
                for (int j = 0; j < NumNodes-1; j++)
                {


                    Problems = new BipartitioningProblem[3];

                    Problems[0] = new BipartitioningProblem();
                    Problems[1] = new BipartitioningProblem();
                    Problems[2] = new BipartitioningProblem();

                    Problems[0].CalcType = CalcType.Neighbor;
                    Problems[1].CalcType = CalcType.Neighbor;
                    Problems[2].CalcType = CalcType.Neighbor;

                    /*
                                        Problems[0].ExceptCoef = (float)0.8;
                                        Problems[1].ExceptCoef = (float)0.6;
                                        Problems[2].ExceptCoef = (float)0.4;
                                        */

                    /*
                    Problems[0].ExpLength =16;
                    Problems[1].ExpLength = 12;
                    Problems[2].ExpLength = 8;
                    /**/

                    
                                       Problems[0].RegrowSize = 4;
                                       Problems[1].RegrowSize =8;
                                       Problems[2].RegrowSize = 12;
                    /*                    /*                   */
                    s = "a";


                    nNumberOfNodes = aNumberOfNodes[j];
                    nNumberOfSubgraphs = aNumberOfSubgraphs[i];

                    stProblem = Folder + "\\Sol\\Pro" + nNumberOfSubgraphs + "X" + nNumberOfNodes;




                    s = PrintStatsCompare(stProblem, Problems, 3);
                    //                    stProblem = 



                    fileSolutions = new System.IO.StreamWriter("testCompare.txt", true);

                    fileSolutions.WriteLine(nNumberOfSubgraphs + "X" + nNumberOfNodes + " & " + s);
                    fileSolutions.Close();




                }
            }

        }


        public void StatsMatPower(string Folder)
        {



       

            BipartitioningProblem[] Problems = new BipartitioningProblem[2];

        

            //            Problems[0].ExceptCoef = (float)0.6;
            //            Problems[1].ExceptCoef = (float)0.8;
            //            Problems[2].CalcType = CalcType.Neighbor;

            string s;
            string stProblem;

            List<int> Solutions = new List<int>();
            List<int> Opt = new List<int>();
            List<int> Iter = new List<int>();
            List<long> CalcTime = new List<long>();

            System.IO.StreamWriter fileSolutions;


            GeneratePowerMatFileNames();
            BipartitioningProblem[] tProblem = new BipartitioningProblem[2];





            for (int i = 0; i < 2; i++)
            {
                tProblem[i] = new BipartitioningProblem();
                tProblem[i].ExceptCoef = (float)0.6;

            }
            fileSolutions = new System.IO.StreamWriter("testMathc.txt");


            fileSolutions.Close();


            for (int i = 0; i < mNumberOfPowerMatFileNames; i++) { 



            //            ABSBipartitioning.ExceptCoef = (float)0.5;



            BFSNodeContainer.InitStatic(10000);






                    Problems = new BipartitioningProblem[2];

                    Problems[0] = new BipartitioningProblem();
                    Problems[1] = new BipartitioningProblem();
                    //                    Problems[2] = new BipartitioningProblem();

                    Problems[0].CalcType = CalcType.Random;
                    Problems[1].CalcType = CalcType.Neighbor;
                    s = "a";



                stProblem = Folder + "\\Sol\\" + mPowerMatFileNames[i];
                    



                    s = PrintStatsMatPower(stProblem, Problems, 2);
                    //                    stProblem = 



                    fileSolutions = new System.IO.StreamWriter("testMathc.txt", true);

                    fileSolutions.WriteLine(mPowerMatFileNames[i] + " & " + s);
                    fileSolutions.Close();




              
            }

        }

        private void buttonSolveAll_Click(object sender, EventArgs e)
        {

            string Folder = ".\\Basic\\Alpha2\\";
            string[] Folders;
            //            TestNeighbor(Folder);
            mProblem = new BipartitioningProblem();
            mProblem.ExceptCoef = (float)0.6;
            mProblem.CalcType = CalcType.Neighbor;
            //            mProblem.ExpLength = 12;
            //            mProblem.RegrowSize = 8;

            //            SolvePowerMatSet();

//            mProblem = new BipartitioningProblem();
//            mProblem.ExceptCoef = (float)0.6;
//            mProblem.CalcType = CalcType.Random;

//            CalcStandard(Folder, mProblem, NumGraphs-1, NumGraphs, 0, NumNodes);
//            CalcStandard(Folder, mProblem, 0, NumGraphs-1, NumNodes-1, NumNodes);


            Folders = new string[2];

            Folders[0] = ".\\Basic\\Alpha2\\";
            Folders[1] = ".\\Basic\\Alpha15\\";


            for (int i = 1; i < 2; i++)
            {
                Folder = Folders[i];

                mProblem = new BipartitioningProblem();
                mProblem.CalcType = CalcType.Random;
                CalcStandard(Folder, mProblem, NumGraphs - 1, NumGraphs, 0, NumNodes);
                CalcStandard(Folder, mProblem, 0, NumGraphs - 1, NumNodes - 1, NumNodes);


                mProblem = new BipartitioningProblem();
                mProblem.CalcType = CalcType.Neighbor;
                CalcStandard(Folder, mProblem, NumGraphs - 1, NumGraphs, 0, NumNodes);
                CalcStandard(Folder, mProblem, 0, NumGraphs - 1, NumNodes - 1, NumNodes);
                /*
                                mProblem = new BipartitioningProblem();
                                mProblem.ExceptCoef = (float)0.4;
                                mProblem.CalcType = CalcType.Neighbor;
                                CalcStandard(Folder, mProblem, NumGraphs - 1, NumGraphs, 0, NumNodes);
                                CalcStandard(Folder, mProblem, 0, NumGraphs - 1, NumNodes - 1, NumNodes);

                                mProblem = new BipartitioningProblem();
                                mProblem.ExceptCoef = (float)0.8;
                                mProblem.CalcType = CalcType.Neighbor;
                                CalcStandard(Folder, mProblem, NumGraphs - 1, NumGraphs, 0, NumNodes);
                                CalcStandard(Folder, mProblem, 0, NumGraphs - 1, NumNodes - 1, NumNodes);


                                mProblem = new BipartitioningProblem();
                //                mProblem.ExceptCoef = (float)0.6;
                                mProblem.ExpLength = 8;
                                mProblem.CalcType = CalcType.Neighbor;
                                CalcStandard(Folder, mProblem, 0, NumGraphs, 0, NumNodes);


                                mProblem = new BipartitioningProblem();
                //                mProblem.ExceptCoef = (float)0.6;
                                mProblem.ExpLength = 16;
                                mProblem.CalcType = CalcType.Neighbor;
                                CalcStandard(Folder, mProblem, 0, NumGraphs,0, NumNodes);
                                */
                                /*
                mProblem = new BipartitioningProblem();
//                mProblem.ExceptCoef = (float)0.6;
                mProblem.RegrowSize = 4;
                mProblem.CalcType = CalcType.Neighbor;
                CalcStandard(Folder, mProblem, 0, NumGraphs-1, 0, NumNodes-1);


                mProblem = new BipartitioningProblem();
//                mProblem.ExceptCoef = (float)0.6;
                mProblem.RegrowSize = 12;
                mProblem.CalcType = CalcType.Neighbor;
                CalcStandard(Folder, mProblem, 0, NumGraphs-1, 0, NumNodes-1);
                */
            }






            /* Folder = ".\\Basic\\Alpha15\\";
             TestNeighbor(Folder);
             */

        }
        void LoadSolutionData(string Problem, BipartitioningProblem Problems, out List<int> Solutions ,out List<int> Iters, out List<long> Times, out List<int> Opt) {


            string FileName;
            System.IO.StreamReader File;
            string cLine;
            char[] Sep = { ' ', '\t' };
            string[] Vals;


            Solutions = new List<int>();
            Iters = new List<int>();
            Opt = new List<int>();
            Times = new List<long>();

            FileName = Problem + Problems.GetNameCalc() + ".txt";
            File = new System.IO.StreamReader(FileName);

            
            while (!File.EndOfStream)
            {
                cLine = File.ReadLine();
                Vals = cLine.Split(Sep);

                Solutions.Add(Convert.ToInt32(Vals[0]));
                Iters.Add(Convert.ToInt32(Vals[1]));
                Times.Add(Convert.ToInt64(Vals[2]));
                Opt.Add(Convert.ToInt32(Vals[3]));


            }


        }
        string  PrintStats(string Problem, BipartitioningProblem[] Problems, int NumberOfSolvers) {

            System.IO.StreamReader File;
            char[] Sep = { ' ', '\t' };
            List<int>[] Solutions = new List<int>[NumberOfSolvers];
            List<long>[] Times = new List<long>[NumberOfSolvers];
            List<int>[] Iter = new List<int>[NumberOfSolvers];
            List<int>[] Opt = new List<int>[NumberOfSolvers];

            double[] AvgErrorP = new double[NumberOfSolvers];
            double[] MaxErrorP = new double[NumberOfSolvers];
            double[] StdevP = new double[NumberOfSolvers];
            double[] cTime = new double[NumberOfSolvers];
            int[] IsOpt = new int[NumberOfSolvers];
            int numOpt;


            int[] MinIns = new int[40];
            
            double[] Avg = new double[NumberOfSolvers];
            double[] AvgIter = new double[NumberOfSolvers];
            int[] Best = new int[NumberOfSolvers];


            
            String sString;


            double StdevIter;



            for (int i = 0; i < NumberOfSolvers; i++) {
                LoadSolutionData(Problem, Problems[i], out Solutions[i], out Iter[i], out Times[i], out Opt[i]);

                GetStats(Solutions[i], Opt[i], out AvgErrorP[i], out MaxErrorP[i], out StdevP[i]);
                
                numOpt = 0;
                for (int m = 0; m < Solutions[i].Count; m++)
                {

                    if (Solutions[i][m] - Opt[i][m] == 0)
                        numOpt++;
                }


                IsOpt[i] = numOpt;
                AvgIter[i] = Iter[i].Average();


            }


            double bothtails=0;
            double lefttail=0;
            double righttail=0;

            double[] Tails = new double[2];
            double[] DataX = new double[Solutions[0].Count];
            double[] DataY = new double[Solutions[0].Count];
            double[] DataDif = new double[Solutions[0].Count];

            for (int i = 0; i < Solutions[0].Count; i++) {
                DataX[i] = Solutions[0][i];
                DataY[i] = Solutions[1][i];
                DataDif[i] = DataX[i] - DataY[i];
            }

            alglib.studentttests.unequalvariancettest(DataX, Solutions[0].Count, DataY, Solutions[0].Count, ref bothtails,
            ref Tails[0],
            ref Tails[1]);

            alglib.wsr.wilcoxonsignedranktest(DataDif, Solutions[0].Count, 0, ref bothtails, ref Tails[0], ref Tails[1]);


                sString = " ";

            for (int k = 0; k < NumberOfSolvers; k++)
            {
                //                        if(k!= 1)
                if ((Tails[NumberOfSolvers - k-1] < 0.05))
                {
                    sString += "\\underline{"+Math.Abs(AvgErrorP[k]).ToString("0.00") + "}(" + Math.Abs(StdevP[k]).ToString("0.00") + ")" + " & ";
                }
                else {
                    sString += Math.Abs(AvgErrorP[k]).ToString("0.00") + "(" + Math.Abs(StdevP[k]).ToString("0.00") + ")" + " & ";
                }
                //                        else
                //                            sString += "("+Math.Abs(AvgErrorP[k]).ToString("0.00") + ")";

            }
            for (int k = 0; k < NumberOfSolvers; k++)
            {
                sString += Math.Abs(MaxErrorP[k]).ToString("0.00") + " & ";
            }
            for (int k = 0; k < NumberOfSolvers; k++)
            {
                sString += Math.Abs(IsOpt[k]).ToString();
                //                        if (k < 1)
                sString += " & ";
            }


            for (int k = 0; k < NumberOfSolvers; k++)
            {

                StdevIter = 0;
                for (int ij = 0; ij < Solutions.Count(); ij++)
                {

                    StdevIter += (Iter[k][ij] - AvgIter[k]) * (Iter[k][ij] - AvgIter[k]);
                }

                StdevIter = Math.Sqrt(StdevIter / (float)Solutions.Count());

                sString += Math.Abs(AvgIter[k]).ToString("0") + "(" + Math.Abs(StdevIter).ToString("0") + ")" + " & ";
                //                        sString += Math.Abs(AvgIter[k]).ToString("0") + "(" + Math.Abs(StdevIter).ToString("0.0") + ")" + " & ";
                //                        else
                //                            sString += "("+Math.Abs(AvgErrorP[k]).ToString("0.00") + ")";

            }

            for (int k = 0; k < NumberOfSolvers; k++)
            {
                sString += (Times[k].Average()).ToString("0");
                if (k < NumberOfSolvers-1)
                    sString += " & ";
            }

            sString += "\\\\";

            return sString;
        }

        string PrintStatsCompare(string Problem, BipartitioningProblem[] Problems, int NumberOfSolvers)
        {

            System.IO.StreamReader File;
            char[] Sep = { ' ', '\t' };
            List<int>[] Solutions = new List<int>[NumberOfSolvers];
            List<long>[] Times = new List<long>[NumberOfSolvers];
            List<int>[] Iter = new List<int>[NumberOfSolvers];
            List<int>[] Opt = new List<int>[NumberOfSolvers];

            double[] AvgErrorP = new double[NumberOfSolvers];
            double[] MaxErrorP = new double[NumberOfSolvers];
            double[] StdevP = new double[NumberOfSolvers];
            double[] cTime = new double[NumberOfSolvers];
            int[] IsOpt = new int[NumberOfSolvers];
            int numOpt;


            int[] MinIns = new int[40];

            double[] Avg = new double[NumberOfSolvers];
            double[] AvgIter = new double[NumberOfSolvers];
            int[] Best = new int[NumberOfSolvers];



            String sString;


            double StdevIter;



            for (int i = 0; i < NumberOfSolvers; i++)
            {
                LoadSolutionData(Problem, Problems[i], out Solutions[i], out Iter[i], out Times[i], out Opt[i]);

                GetStats(Solutions[i], Opt[i], out AvgErrorP[i], out MaxErrorP[i], out StdevP[i]);

            

                AvgIter[i] = Iter[i].Average();


            }


            double bothtails = 0;
            double lefttail = 0;
            double righttail = 0;

            double[] TailsXY = new double[2];
            double[] TailsXZ = new double[2];
            double[] TailsYZ = new double[2];
            double[] DataX = new double[Solutions[0].Count];
            double[] DataY = new double[Solutions[0].Count];
            double[] DataZ = new double[Solutions[0].Count];
            double[] DataDifXY = new double[Solutions[0].Count];
            double[] DataDifXZ = new double[Solutions[0].Count];
            double[] DataDifYZ = new double[Solutions[0].Count];

            for (int i = 0; i < Solutions[0].Count; i++)
            {
                DataX[i] = Solutions[0][i];
                DataY[i] = Solutions[1][i];
                DataZ[i] = Solutions[2][i];
                DataDifXY[i] = DataX[i] - DataY[i];
                DataDifXZ[i] = DataX[i] - DataZ[i];
                DataDifYZ[i] = DataY[i] - DataZ[i];
            }


            alglib.wsr.wilcoxonsignedranktest(DataDifXY, Solutions[0].Count, 0, ref bothtails, ref TailsXY[0], ref TailsXY[1]);
            alglib.wsr.wilcoxonsignedranktest(DataDifXZ, Solutions[0].Count, 0, ref bothtails, ref TailsXZ[0], ref TailsXZ[1]);
            alglib.wsr.wilcoxonsignedranktest(DataDifYZ, Solutions[0].Count, 0, ref bothtails, ref TailsYZ[0], ref TailsYZ[1]);


            sString = " ";

            for (int k = 0; k < NumberOfSolvers; k++)
            {
                //                        if(k!= 1)
                sString += Math.Abs(AvgErrorP[k]).ToString("0.00") + "(" + Math.Abs(StdevP[k]).ToString("0.00") + ")" + " & ";
                //                        else
                //                            sString += "("+Math.Abs(AvgErrorP[k]).ToString("0.00") + ")";

            }


            for (int k = 0; k < NumberOfSolvers; k++)
            {
                sString += (Times[k].Average()).ToString("0");
                    sString += " & ";
            }
            /*      */

            if ((TailsXY[0] < 0.05) || (TailsXY[1] < 0.05))
                sString += " Y ";
            else 
                sString += " N ";
            sString += " & ";

            if ((TailsXZ[0] < 0.05) || (TailsXZ[1] < 0.05))
                sString += " Y ";
            else
                sString += " N ";
            sString += " & ";

            if ((TailsYZ[0] < 0.05) || (TailsYZ[1] < 0.05))
                sString += " Y ";
            else
                sString += " N ";


            sString += "\\\\";

            return sString;
        }



        string PrintStatsMatPower(string Problem, BipartitioningProblem[] Problems, int NumberOfSolvers)
        {

            System.IO.StreamReader File;
            char[] Sep = { ' ', '\t' };
            List<int>[] Solutions = new List<int>[NumberOfSolvers];
            List<long>[] Times = new List<long>[NumberOfSolvers];
            List<int>[] Iter = new List<int>[NumberOfSolvers];
            List<int>[] Opt = new List<int>[NumberOfSolvers];

            double[] AvgErrorP = new double[NumberOfSolvers];
            double[] MaxErrorP = new double[NumberOfSolvers];
            double[] StdevP = new double[NumberOfSolvers];
            double[] cTime = new double[NumberOfSolvers];
            int[] IsOpt = new int[NumberOfSolvers];
            int numOpt;


            int[] MinIns = new int[40];

            double[] Avg = new double[NumberOfSolvers];
            double[] AvgIter = new double[NumberOfSolvers];
            int[] Best = new int[NumberOfSolvers];



            String sString;


            double StdevIter;



            for (int i = 0; i < NumberOfSolvers; i++)
            {
                LoadSolutionData(Problem, Problems[i], out Solutions[i], out Iter[i], out Times[i], out Opt[i]);

                //                GetStats(Solutions[i], Opt[i], out AvgErrorP[i], out MaxErrorP[i], out StdevP[i]);
                GetStatsSimp(Solutions[i], out AvgErrorP[i], out MaxErrorP[i], out StdevP[i]);
                numOpt = 0;
                for (int m = 0; m < Solutions[i].Count; m++)
                {

                    if (Solutions[i][m] - Opt[i][m] == 0)
                        numOpt++;
                }


                IsOpt[i] = numOpt;
                AvgIter[i] = Iter[i].Average();


            }


            double bothtails = 0;
            double lefttail = 0;
            double righttail = 0;

            double[] Tails = new double[2];
            double[] DataX = new double[Solutions[0].Count];
            double[] DataY = new double[Solutions[0].Count];
            double[] DataDif = new double[Solutions[0].Count];

            for (int i = 0; i < Solutions[0].Count; i++)
            {
                DataX[i] = Solutions[0][i];
                DataY[i] = Solutions[1][i];
                DataDif[i] = DataX[i] - DataY[i];
            }

            alglib.studentttests.unequalvariancettest(DataX, Solutions[0].Count, DataY, Solutions[0].Count, ref bothtails,
            ref Tails[0],
            ref Tails[1]);

//            alglib.wsr.wilcoxonsignedranktest(DataDif, Solutions[0].Count, 0, ref bothtails, ref Tails[0], ref Tails[1]);


            sString = " ";

            for (int k = 0; k < NumberOfSolvers; k++)
            {
                //                        if(k!= 1)
                if ((Tails[NumberOfSolvers - k - 1] < 0.05))
                {
                    sString += "\\underline{" + Math.Abs(AvgErrorP[k]).ToString("0.00") + "}(" + Math.Abs(StdevP[k]).ToString("0.00") + ")" + " & ";
                }
                else
                {
                    sString += Math.Abs(AvgErrorP[k]).ToString("0.00") + "(" + Math.Abs(StdevP[k]).ToString("0.00") + ")" + " & ";
                }
                //                        else
                //                            sString += "("+Math.Abs(AvgErrorP[k]).ToString("0.00") + ")";

            }
            for (int k = 0; k < NumberOfSolvers; k++)
            {
                sString += Math.Abs(MaxErrorP[k]).ToString("0") + " & ";
            }

            /*
            for (int k = 0; k < NumberOfSolvers; k++)
            {
                sString += Math.Abs(IsOpt[k]).ToString();
                //                        if (k < 1)
                sString += " & ";
            }
            */

     /*       for (int k = 0; k < NumberOfSolvers; k++)
            {

                StdevIter = 0;
                for (int ij = 0; ij < Solutions.Count(); ij++)
                {

                    StdevIter += (Iter[k][ij] - AvgIter[k]) * (Iter[k][ij] - AvgIter[k]);
                }

                StdevIter = Math.Sqrt(StdevIter / (float)Solutions.Count());

                sString += Math.Abs(AvgIter[k]).ToString("0") + "(" + Math.Abs(StdevIter).ToString("0") + ")" + " & ";
                //                        sString += Math.Abs(AvgIter[k]).ToString("0") + "(" + Math.Abs(StdevIter).ToString("0.0") + ")" + " & ";
                //                        else
                //                            sString += "("+Math.Abs(AvgErrorP[k]).ToString("0.00") + ")";

            }
            */
            for (int k = 0; k < NumberOfSolvers; k++)
            {
                sString += Times[k].Average().ToString("0");
                if (k < NumberOfSolvers - 1)
                    sString += " & ";
            }

            sString += "\\\\";

            return sString;
        }
        void SolveNeighborParamExcept(string Folder) {

            double[] Except = { 0.4, 0.6, 0.8 };
            for (int i = 0; i < 3; i++) {

                mProblem = new BipartitioningProblem();
                mProblem.ExceptCoef = (float)0.6;
                mProblem.CalcType = CalcType.Neighbor;
       //         CalcStandard(Folder, mProblem);
            }

        }


        void CalcPowerMat(string PowerMatFileName, BipartitioningProblem Problem)
        {

            GeneratePowerMatFileNames();
            
            int NumTests = 10;
            int[,,] results = new int[21, 2, NumTests];

            List<int> Solutions = new List<int>();
            List<int> Opt = new List<int>();
            List<int> Iter = new List<int>();
            List<long>CalcTime = new List<long>();








                    

            System.IO.StreamWriter fileSolutions;

            Problem.MultipleSolveMatPower(PowerMatFileName, NumTests, out Solutions, out Opt, out Iter, out CalcTime);


            fileSolutions = new System.IO.StreamWriter(".\\Rw\\Sol\\"+PowerMatFileName + Problem.GetNameCalc() + ".txt");

            for (int i = 0; i < Solutions.Count; i++) {
                fileSolutions.WriteLine(Solutions[i] + " " + Iter[i] + " " + CalcTime[i] + " -1");
            }
            fileSolutions.Close();



             


        }


        void ConvertMatPowerToProblemInstance(string FileName) {

            ABSGraph tGraph = new ABSGraph();
            int NumberOfRoots;
            int NumberOfNodesInSubgraph;
            bool[] IsInRoot;
            System.IO.StreamWriter File = new System.IO.StreamWriter(".\\RW\\" + FileName + ".pro");
            int counter = 0;
            int cIndex;
            List<int> Calc = new List<int>();   

            tGraph.LoadPowerMat(".\\RW\\" + FileName + ".txt");

            IsInRoot = new bool[tGraph.NumberOfVertices];

            for (int i = 0; i < tGraph.NumberOfVertices; i++)
            {

                IsInRoot[i] = false;
            }

            if (tGraph.NumberOfVertices > 500)
            {
                NumberOfRoots = (int)Math.Ceiling(tGraph.NumberOfVertices / (float)200);
                NumberOfNodesInSubgraph = 200;

            }
            else
            {
                NumberOfRoots = (int)Math.Ceiling(tGraph.NumberOfVertices / (float)50);
                NumberOfNodesInSubgraph = 50;
            }


            while (counter < NumberOfRoots)
            {

                cIndex = mGenerator.Next() % tGraph.NumberOfVertices;
                mProblem.SetGraph(tGraph);

                
                if (!IsInRoot[cIndex])
                {
                    IsInRoot[cIndex] = true;

//                    counter++;


                    mProblem.MaxNodesInSubgraph = NumberOfNodesInSubgraph;

                    mProblem.InitTest(counter+1, tGraph, IsInRoot);
                    mProblem.NumberOfSubgraphs = counter+1;
                    
                    
                    
                    mProblem.ClearBest();
                    mProblem.Partitioning.CreateAllSubgraphs();
                    mProblem.Partitioning.RestartSubgraph(counter);
                    Calc.Clear();
                    Calc.Add(counter);
                    mProblem.Partitioning.StartParitial(counter, Calc);

                    mProblem.SetAllMaxNodesInCycle();
                    mProblem.Start();
                    mProblem.MultiStepSeq(158000, mGenerator.Next() % (12) + 2, Calc);

                    if (!(mProblem.Partitioning.GetSubgraph(counter).NumberOfCycleNodes()>1))
                    {

//                        counter--;
                        IsInRoot[cIndex] = false;
                    }
                    else {
                    
                        counter++;
                    }
                    
                }
            }


            File.WriteLine("Number Of Nodes");

            File.WriteLine(tGraph.NumberOfVertices);


            File.WriteLine("Number Of Subgraphs");
            File.WriteLine(NumberOfRoots);
            File.WriteLine("Number Of Nodes In Subgraphs");
            File.WriteLine(NumberOfNodesInSubgraph);
            File.WriteLine("Roots ");
            for (int i = 0; i < tGraph.NumberOfVertices; i++)
            {

                if (IsInRoot[i])
                    File.WriteLine(i);
            }

            File.WriteLine("Graph");
            tGraph.Save(File);

            File.Close();

        }
        void GeneratePowerMatFileNames() {

            int i = 0;
            mPowerMatFileNames = new string[21];

            mPowerMatFileNames[i++] = "case118";
            mPowerMatFileNames[i++] = "case145";
            mPowerMatFileNames[i++] = "case_illinois200";
//            mPowerMatFileNames[i++] = "case1354pegase";
            mPowerMatFileNames[i++] = "case1888rte";
            mPowerMatFileNames[i++] = "case1951rte";
            mPowerMatFileNames[i++] = "case2383wp";
            mPowerMatFileNames[i++] = "case2736sp";
            mPowerMatFileNames[i++] = "case2737sop";
            mPowerMatFileNames[i++] = "case2746wop";
            mPowerMatFileNames[i++] = "case2746wp";
            mPowerMatFileNames[i++] = "case2848rte";
            mPowerMatFileNames[i++] = "case2868rte";
//            mPowerMatFileNames[i++] = "case2869pegase";
            mPowerMatFileNames[i++] = "case3012wp";
            mPowerMatFileNames[i++] = "case3375wp";
            mPowerMatFileNames[i++] = "case6468rte";
            mPowerMatFileNames[i++] = "case6470rte";
            mPowerMatFileNames[i++] = "case6495rte";
            mPowerMatFileNames[i++] = "case9241pegase";
//            mPowerMatFileNames[i++] = "case13659pegase";

            mNumberOfPowerMatFileNames = i;
        }

        void SolvePowerMatSet() {

            GeneratePowerMatFileNames();
            BipartitioningProblem[] tProblem = new BipartitioningProblem[2];
           
         



            for (int i = 0; i < 2; i++) {
                tProblem[i] = new BipartitioningProblem();
                tProblem[i].ExceptCoef = (float)0.6;

            }

            tProblem[0].CalcType = CalcType.Random;
            tProblem[1].CalcType = CalcType.Neighbor;

//            System.IO.StreamWriter File = new System.IO.StreamWriter("testMatPower.txt");

//            File.Close();

            for (int i = 0; i < mNumberOfPowerMatFileNames; i++)
            {

                for (int j = 0; j < 2; j++)
                {
                    CalcPowerMat(mPowerMatFileNames[i], tProblem[j]);
                }
                
            }
            

        }

        private void buttonGenPowMat_Click(object sender, EventArgs e)
        {

            GeneratePowerMatFileNames();


            for (int i = 0; i < mNumberOfPowerMatFileNames; i++) {

                ConvertMatPowerToProblemInstance(mPowerMatFileNames[i]);


            }
           
        }

        private void buttonSolvePowerMat_Click(object sender, EventArgs e)
        {
//            ABSBipartitioning.ExceptCoef = (float)0.6;
            SolvePowerMatSet();
        }

        private void buttonCompareGraph_Click(object sender, EventArgs e)
        {
            GeneratePowerMatFileNames();


            string FileName = ".\\RW\\" + mPowerMatFileNames[7] + ".pro";

            FileName = ".\\Basic\\Alpha2\\prob_50_100_1.pro";
            mProblem.Load(FileName);

            double[] ExceptCoef = { 0.4, 0.6, 0.8 };

            for (int i = 0; i < 3; i++)
            {
                mProblem.ExceptCoef = (float)ExceptCoef[i];
                mProblem.CalcType = CalcType.Neighbor;
                mProblem.SetAllMaxNodesInCycle();
                //                    tProblem[j].CalcType = CalcType.Neighbor;
                mProblem.NewGenerator(0);
                mProblem.Start();
                mProblem.ClearBest();
                mProblem.FindSolutionPartial(15000, 1);
                mProblem.SaveIntermediateSolutions(FileName);
            }

        }

        private void buttonGetStatsStandard_Click(object sender, EventArgs e)
        {
            BipartitioningProblem[] Problems = new BipartitioningProblem[2];

            Problems[0] = new BipartitioningProblem();
            Problems[1] = new BipartitioningProblem();

            Problems[0].CalcType = CalcType.Neighbor;
            Problems[1].CalcType = CalcType.Random;

            string s = PrintStats(".\\Basic\\Alpha2\\Sol\\Pro50x5", Problems, 2);


            System.IO.StreamWriter fileSolutions = new System.IO.StreamWriter("testc.txt");

            fileSolutions.WriteLine(s);
            fileSolutions.Close();

            StatsStandard(".\\Basic\\Alpha15");
            StatsStandardCompare(".\\Basic\\Alpha2");

            StatsMatPower(".\\RW");

        }
    }
}
