﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace HamiltoninaGSD
{
    class ProblemInstance
    {
        ABSGraph mGraph;
        Kord[] mKords;
        List<List<int>> mParitions;
        int[] mRoots;

        int mNumberOfSubgraphs;
        int mNumberOfNodesInSubgraphs;
        int mNumberOfNodes;


        public Kord[] Kords{
            
                    get{ return mKords; }
            }

        public List<Kord> GetPartitionKords(int Index)
        {
            List<Kord> Result = new List<Kord>();

            foreach (int i in mParitions[Index]) {

                Result.Add(new Kord(Kords[i]));
            }

            return Result;
            
        }

        public ProblemInstance(Kord[] nKords, List<List<int>> nPartitions, int[] nRoots, int nNumberOfSubgraphs, int nNumberOfNodesInSubgraphs, double R) {

            mKords =  nKords;
            mParitions = nPartitions;
            mNumberOfNodesInSubgraphs = nNumberOfNodesInSubgraphs;
            mNumberOfSubgraphs = nNumberOfSubgraphs;
            mGraph = new ABSGraph();
            mRoots = nRoots;
            mGraph.GenerateUnitDisc(mKords, R, mNumberOfSubgraphs * mNumberOfNodesInSubgraphs);
            mNumberOfNodes = mNumberOfNodesInSubgraphs * mNumberOfSubgraphs;
        }

        public void SaveKords(String FileName)
        {

            System.IO.StreamWriter File = new StreamWriter(FileName);
            SaveKords(File);

            File.Close();

        }


        public void SaveKords(System.IO.StreamWriter File) {
            File.WriteLine("Number Of Vertices");
            File.WriteLine(mNumberOfNodes);
            File.WriteLine("Node Positions X Y");

            for (int i = 0; i < mNumberOfNodesInSubgraphs * mNumberOfSubgraphs; i++)
                File.WriteLine(mKords[i].mX + " " + mKords[i].mY);
        }


        public void SaveRoots(System.IO.StreamWriter File)
        {
            File.WriteLine("Roots ");

            for (int i = 0; i < mNumberOfSubgraphs; i++)
                File.WriteLine(mRoots[i]);
        }

        public void SavePartitions(String FileName)
        {

            System.IO.StreamWriter File = new StreamWriter(FileName);
            SaveParititions(File);

            File.Close();

        }
        public void SaveParititions(System.IO.StreamWriter File)
        {
            int counter = 0;
            File.WriteLine("Number Of Subgraphs");
            File.WriteLine(mNumberOfSubgraphs);
            File.WriteLine("Number Of Nodes In Subgraphs");
            File.WriteLine(mNumberOfNodesInSubgraphs );

            counter = 0;
            foreach (List<int> l in mParitions) {
                File.WriteLine("PARTITION " + counter++);
                File.WriteLine("Nodes Inside ");
                File.WriteLine(l.Count());
                File.WriteLine("list");

                foreach (int a in l) {
                    File.WriteLine(a);
                }

            }
            
        }

        public void SaveProblem(String FileName)
        {

            System.IO.StreamWriter File = new StreamWriter(FileName);
            SaveProblem(File);

            File.Close();

        }
        public void SaveProblem(System.IO.StreamWriter File)
        {
            File.WriteLine("Number Of Nodes");

            File.WriteLine(mNumberOfNodes);
            File.WriteLine("Number Of Subgraphs");
            File.WriteLine(mNumberOfSubgraphs);
            File.WriteLine("Number Of Nodes In Subgraphs");
            File.WriteLine(mNumberOfNodesInSubgraphs);
            SaveRoots(File);
            File.WriteLine("Graph");
            mGraph.Save(File);


        }

        public void SaveComplete(String FileNameStart) {

            SaveProblem(FileNameStart + ".pro");
            SavePartitions(FileNameStart + ".sol");
            SaveKords(FileNameStart + ".vis");
        }
    }
}
